"""Test variables"""

# pylint: disable=C0103, C0302

client_config = (
    '---\n'
    'elasticsearch:\n'
    '  client:\n'
    '    hosts: {0}\n'
    '    request_timeout: 30\n'
    '  other_settings:\n'
    '    master_only: False\n'
    '\n'
    'logging:\n'
    '  loglevel: DEBUG\n'
    '  logfile:\n'
    '  logformat: default\n'
    '  blacklist: []\n'
)

client_conf_logfile = (
    '---\n'
    'elasticsearch:\n'
    '  client:\n'
    '    hosts: {0}\n'
    '\n'
    'logging:\n'
    '  loglevel: DEBUG\n'
    '  logfile: {1}\n'
)

client_config_envvars = (
    '---\n'
    'elasticsearch:\n'
    '  client:\n'
    '    hosts: {0}\n'
    '    request_timeout: {1}\n'
    '  other_settings:\n'
    '    master_only: False\n'
    '\n'
    'logging:\n'
    '  loglevel: DEBUG\n'
    '  logfile:\n'
    '  logformat: default\n'
    '  blacklist: []\n'
)

bad_client_config = (
    '---\n'
    'elasticsearch:\n'
    '  misspelled:\n'
    '    hosts: {0}\n'
    '    certificate:\n'
    '    client_cert:\n'
    '    client_key:\n'
    '    request_timeout: 30\n'
    '  other_settings:\n'
    '    master_only: False\n'
)

no_logging_config = (
    '---\n'
    'elasticsearch:\n'
    '  client:\n'
    '    hosts: {0}\n'
    '    certificate:\n'
    '    client_cert:\n'
    '    client_key:\n'
    '    request_timeout: 30\n'
    '  other_settings:\n'
    '    master_only: False\n'
)

none_logging_config = (
    '---\n'
    'elasticsearch:\n'
    '  client:\n'
    '    hosts: {0}\n'
    '    certificate:\n'
    '    client_cert:\n'
    '    client_key:\n'
    '    request_timeout: 30\n'
    '  other_settings:\n'
    '    master_only: False\n'
    '\n'
    'logging: \n'
    '\n'
)

alias_add_only = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Add all indices to specified alias"\n'
    '    action: alias\n'
    '    options:\n'
    '      name: {0}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    add:\n'
    '      filters:\n'
    '        - filtertype: none\n'
)

alias_add_only_with_extra_settings = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Add all indices to specified alias"\n'
    '    action: alias\n'
    '    options:\n'
    '      name: {0}\n'
    '      extra_settings:\n'
    '        filter:\n'
    '          term:\n'
    '            user: kimchy\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    add:\n'
    '      filters:\n'
    '        - filtertype: none\n'
)

alias_remove_only = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Remove all indices from specified alias"\n'
    '    action: alias\n'
    '    options:\n'
    '      name: {0}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    remove:\n'
    '      filters:\n'
    '        - filtertype: none\n'
)

alias_add_remove = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Add/remove specified indices from designated alias"\n'
    '    action: alias\n'
    '    options:\n'
    '      name: {0}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    remove:\n'
    '      filters:\n'
    '        - filtertype: pattern\n'
    '          kind: prefix\n'
    '          value: du\n'
    '    add:\n'
    '      filters:\n'
    '        - filtertype: pattern\n'
    '          kind: prefix\n'
    '          value: my\n'
)

alias_remove_index_not_there = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Add/remove specified indices from designated alias"\n'
    '    action: alias\n'
    '    options:\n'
    '      name: {0}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    remove:\n'
    '      filters:\n'
    '        - filtertype: pattern\n'
    '          kind: prefix\n'
    '          value: {1}\n'
)

alias_add_with_empty_remove = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Add/remove specified indices from designated alias"\n'
    '    action: alias\n'
    '    options:\n'
    '      name: {0}\n'
    '      warn_if_no_indices: True\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    remove:\n'
    '      filters:\n'
    '        - filtertype: pattern\n'
    '          kind: prefix\n'
    '          value: insertrickrollhere\n'
    '    add:\n'
    '      filters:\n'
    '        - filtertype: pattern\n'
    '          kind: prefix\n'
    '          value: my\n'
)

alias_remove_with_empty_add = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Add/remove specified indices from designated alias"\n'
    '    action: alias\n'
    '    options:\n'
    '      name: {0}\n'
    '      warn_if_no_indices: True\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    remove:\n'
    '      filters:\n'
    '        - filtertype: pattern\n'
    '          kind: prefix\n'
    '          value: du\n'
    '    add:\n'
    '      filters:\n'
    '        - filtertype: pattern\n'
    '          kind: prefix\n'
    '          value: insertrickrollhere\n'
)

alias_add_remove_empty = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Add/remove specified indices from designated alias"\n'
    '    action: alias\n'
    '    options:\n'
    '      name: {0}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    remove:\n'
    '      filters:\n'
    '        - filtertype: pattern\n'
    '          kind: prefix\n'
    '          value: {1}\n'
    '    add:\n'
    '      filters:\n'
    '        - filtertype: pattern\n'
    '          kind: prefix\n'
    '          value: {2}\n'
)

alias_no_add_remove = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "No add or remove should raise an exception"\n'
    '    action: alias\n'
    '    options:\n'
    '      name: {0}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
)

alias_no_alias = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Removing alias from options should result in an exception"\n'
    '    action: alias\n'
    '    options:\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    remove:\n'
    '      filters:\n'
    '        - filtertype: pattern\n'
    '          kind: prefix\n'
    '          value: du\n'
    '    add:\n'
    '      filters:\n'
    '        - filtertype: pattern\n'
    '          kind: prefix\n'
    '          value: my\n'
)

allocation_test = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Allocate by key/value/allocation_type"\n'
    '    action: allocation\n'
    '    options:\n'
    '      key: {0}\n'
    '      value: {1}\n'
    '      allocation_type: {2}\n'
    '      wait_for_completion: {3}\n'
    '      wait_interval: 1\n'
    '      max_wait: -1\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: my\n'
)


allocation_count_test = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Allocate by key/value/allocation_type"\n'
    '    action: allocation\n'
    '    options:\n'
    '      key: {0}\n'
    '      value: {1}\n'
    '      allocation_type: {2}\n'
    '      wait_for_completion: {3}\n'
    '      wait_interval: 1\n'
    '      max_wait: -1\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: count\n'
    '        use_age: True\n'
    '        source: name\n'
    '        timestring: \'%Y.%m.%d\'\n'
    '        count: 2\n'
    '        exclude: False\n'
)

cluster_routing_test = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Alter cluster routing by routing_type/value"\n'
    '    action: cluster_routing\n'
    '    options:\n'
    '      routing_type: {0}\n'
    '      value: {1}\n'
    '      setting: enable\n'
)

optionless_proto = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Act on indices as filtered"\n'
    '    action: {0}\n'
    '    options:\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: my\n'
)

no_options_proto = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Act on indices as filtered"\n'
    '    action: {0}\n'
    '    filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: my\n'
)

actionless_proto = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    options:\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
)

disabled_proto = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Act on indices as filtered"\n'
    '    action: {0}\n'
    '    options:\n'
    '      continue_if_exception: False\n'
    '      ignore_empty_list: True\n'
    '      disable_action: True\n'
    '    filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: my\n'
    '  2:\n'
    '    description: "Act on indices as filtered"\n'
    '    action: {1}\n'
    '    options:\n'
    '      continue_if_exception: False\n'
    '      ignore_empty_list: True\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: log\n'
)

continue_proto = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Create named index"\n'
    '    action: create_index\n'
    '    options:\n'
    '      name: {0}\n'
    '      continue_if_exception: {1}\n'
    '      disable_action: False\n'
    '  2:\n'
    '    description: "Act on indices as filtered"\n'
    '    action: {2}\n'
    '    options:\n'
    '      continue_if_exception: {3}\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: log\n'
)

close_delete_aliases = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Close indices as filtered"\n'
    '    action: close\n'
    '    options:\n'
    '      delete_aliases: True\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: my\n'
)

close_skip_flush = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Close indices as filtered"\n'
    '    action: close\n'
    '    options:\n'
    '      skip_flush: True\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: my\n'
)

close_ignore_sync = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Close indices as filtered"\n'
    '    action: close\n'
    '    options:\n'
    '      ignore_sync_failures: {0}\n'
    '    filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: my\n'
)

delete_proto = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Delete indices as filtered"\n'
    '    action: delete_indices\n'
    '    options:\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: {0}\n'
    '        source: {1}\n'
    '        direction: {2}\n'
    '        timestring: {3}\n'
    '        unit: {4}\n'
    '        unit_count: {5}\n'
    '        field: {6}\n'
    '        stats_result: {7}\n'
    '        epoch: {8}\n'
)

delete_pattern_proto = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Delete indices as filtered"\n'
    '    action: delete_indices\n'
    '    options:\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: {0}\n'
    '        source: {1}\n'
    '        direction: {2}\n'
    '        timestring: {3}\n'
    '        unit: {4}\n'
    '        unit_count: {5}\n'
    '        unit_count_pattern: {6}\n'
)


delete_period_proto = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Delete indices as filtered"\n'
    '    action: delete_indices\n'
    '    options:\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '    - filtertype: pattern\n'
    '      kind: prefix\n'
    '      value: ilm-history-\n'
    '      exclude: True\n'
    '    - filtertype: {0}\n'
    '      source: {1}\n'
    '      range_from: {2}\n'
    '      range_to: {3}\n'
    '      timestring: {4}\n'
    '      unit: {5}\n'
    '      field: {6}\n'
    '      stats_result: {7}\n'
    '      intersect: {8}\n'
    '      epoch: {9}\n'
    '      week_starts_on: {10}\n'
)

delete_ignore_proto = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Delete indices as filtered"\n'
    '    action: delete_indices\n'
    '    options:\n'
    '      ignore_empty_list: True\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: {0}\n'
    '        source: {1}\n'
    '        direction: {2}\n'
    '        timestring: {3}\n'
    '        unit: {4}\n'
    '        unit_count: {5}\n'
    '        field: {6}\n'
    '        stats_result: {7}\n'
    '        epoch: {8}\n'
)

filter_by_alias = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Delete indices as filtered"\n'
    '    action: delete_indices\n'
    '    options:\n'
    '      ignore_empty_list: True\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: alias\n'
    '        aliases: {0}\n'
    '        exclude: {1}\n'
)

filter_closed = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Delete indices as filtered"\n'
    '    action: delete_indices\n'
    '    options:\n'
    '      ignore_empty_list: True\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: closed\n'
    '        exclude: {0}\n'
)

bad_option_proto_test = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Should raise exception due to extra option"\n'
    '    action: {0}\n'
    '    options:\n'
    '      invalid: this_should_not_be_here\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: none\n'
)

bad_option_rollover_test = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Should raise exception due to extra option"\n'
    '    action: {0}\n'
    '    options:\n'
    '      invalid: this_should_not_be_here\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
)

replicas_test = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Increase replica count to provided value"\n'
    '    action: replicas\n'
    '    options:\n'
    '      count: {0}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: my\n'
)

forcemerge_test = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: >-\n'
    '      forceMerge segment count per shard to provided value with optional delay\n'
    '    action: forcemerge\n'
    '    options:\n'
    '      max_num_segments: {0}\n'
    '      delay: {1}\n'
    '      timeout_override: 300\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: my\n'
)

snapshot_test = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Snapshot selected indices"\n'
    '    action: snapshot\n'
    '    options:\n'
    '      repository: {0}\n'
    '      name: {1}\n'
    '      wait_interval: {2}\n'
    '      max_wait: {3}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: none\n'
)

delete_snap_proto = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Delete snapshots as filtered"\n'
    '    action: delete_snapshots\n'
    '    options:\n'
    '      repository: {0}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '      - filtertype: {1}\n'
    '        source: {2}\n'
    '        direction: {3}\n'
    '        timestring: {4}\n'
    '        unit: {5}\n'
    '        unit_count: {6}\n'
    '        epoch: {7}\n'
)

create_index = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Create index as named"\n'
    '    action: create_index\n'
    '    options:\n'
    '      name: {0}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
)

create_index_with_extra_settings = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Create index as named with extra settings"\n'
    '    action: create_index\n'
    '    options:\n'
    '      name: {0}\n'
    '      extra_settings:\n'
    '        aliases:\n'
    '          {1}:\n'
    '            is_write_index: True\n'
    '        settings:\n'
    '          number_of_shards: 1\n'
    '          number_of_replicas: 0\n'
    '        mappings:\n'
    '          properties:\n'
    '            {2}:\n'
    '              type: {3}\n'
    '            {4}:\n'
    '              type: {5}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
)

restore_snapshot_proto = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: Restore snapshot as configured\n'
    '    action: restore\n'
    '    options:\n'
    '      repository: {0}\n'
    '      name: {1}\n'
    '      indices: {2}\n'
    '      include_aliases: {3}\n'
    '      ignore_unavailable: {4}\n'
    '      include_global_state: {5}\n'
    '      partial: {6}\n'
    '      rename_pattern: {7}\n'
    '      rename_replacement: {8}\n'
    '      extra_settings: {9}\n'
    '      wait_for_completion: {10}\n'
    '      skip_repo_fs_check: {11}\n'
    '      timeout_override: {12}\n'
    '      wait_interval: {13}\n'
    '      max_wait: {14}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '    - filtertype: none\n'
)

test_687 = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    action: snapshot\n'
    '    description: >-\n'
    '      Create a snapshot with the last week index.\n'
    '    options:\n'
    '      repository: {0}\n'
    '      name: {1}\n'
    '      ignore_unavailable: False\n'
    '      include_global_state: True\n'
    '      partial: False\n'
    '      wait_for_completion: True\n'
    '      skip_repo_fs_check: False\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '    - filtertype: pattern\n'
    '      kind: prefix\n'
    '      value: logstash-\n'
    '      exclude:\n'
    '    - filtertype: age\n'
    '      source: creation_date\n'
    '      direction: younger\n'
    '      epoch: 1467020729\n'
    '      unit: weeks\n'
    '      unit_count: 2\n'
    '      exclude:\n'
    '    - filtertype: age\n'
    '      source: creation_date\n'
    '      direction: younger\n'
    '      epoch: 1467020729\n'
    '      unit: weeks\n'
    '      unit_count: 1\n'
    '      exclude: True\n'
    '  2:\n'
    '    action: delete_indices\n'
    '    description: >-\n'
    '      Remove indices starting with logstash- older than 5 weeks\n'
    '    options:\n'
    '      ignore_empty_list: True\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '    - filtertype: pattern\n'
    '      kind: prefix\n'
    '      value: logstash-\n'
    '      exclude:\n'
    '    - filtertype: age\n'
    '      source: creation_date\n'
    '      epoch: 1467020729\n'
    '      direction: older\n'
    '      unit: weeks\n'
    '      unit_count: 5\n'
    '      exclude:\n'
)

test_682 = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Snapshot selected indices"\n'
    '    action: snapshot\n'
    '    options:\n'
    '      repository: {0}\n'
    '      name: {1}\n'
    '      ignore_empty_list: {2}\n'
    '      wait_interval: {3}\n'
    '      max_wait: {4}\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '    - filtertype: pattern\n'
    '      kind: prefix\n'
    '      value: notlogstash-\n'
    '      exclude:\n'
    '  2:\n'
    '    description: "Delete selected indices"\n'
    '    action: delete_indices\n'
    '    options:\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '    - filtertype: pattern\n'
    '      kind: prefix\n'
    '      value: logstash-\n'
    '      exclude:\n'
)

CRA_all = {
    'persistent': {},
    'transient': {'cluster': {'routing': {'allocation': {'enable': 'all'}}}},
}

rollover_one = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Rollover selected alias/index"\n'
    '    action: rollover\n'
    '    options:\n'
    '      name: {0}\n'
    '      conditions: \n'
    '        {1}: {2}\n'
    '      extra_settings:\n'
    '        index.number_of_shards: 1\n'
    '        index.number_of_replicas: 0\n'
)

rollover_both = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Rollover selected alias/index"\n'
    '    action: rollover\n'
    '    options:\n'
    '      name: {0}\n'
    '      conditions: \n'
    '        max_age: {1}\n'
    '        max_docs: {2}\n'
    '      extra_settings:\n'
    '        index.number_of_shards: 1\n'
    '        index.number_of_replicas: 0\n'
)

rollover_bad_settings = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Rollover selected alias/index"\n'
    '    action: rollover\n'
    '    options:\n'
    '      name: {0}\n'
    '      conditions: \n'
    '        max_age: {1}\n'
    '        max_docs: {2}\n'
    '      extra_settings:\n'
    '        foo: 1\n'
    '        bar: 0\n'
)

rollover_with_name = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Rollover selected alias/index"\n'
    '    action: rollover\n'
    '    options:\n'
    '      name: {0}\n'
    '      conditions: \n'
    '        {1}: {2}\n'
    '      new_index: {3}\n'
    '      extra_settings:\n'
    '        index.number_of_shards: 1\n'
    '        index.number_of_replicas: 0\n'
)

reindex = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Reindex"\n'
    '    action: reindex\n'
    '    options:\n'
    '      allow_ilm_indices: true\n'
    '      wait_interval: {0}\n'
    '      max_wait: {1}\n'
    '      request_body:\n'
    '        source:\n'
    '          index: {2}\n'
    '        dest:\n'
    '          index: {3}\n'
    '    filters:\n'
    '    - filtertype: none\n'
)

reindex_empty_list = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Reindex"\n'
    '    action: reindex\n'
    '    options:\n'
    '      ignore_empty_list: {0}\n'
    '      wait_interval: {1}\n'
    '      max_wait: {2}\n'
    '      request_body:\n'
    '        source:\n'
    '          index: REINDEX_SELECTED\n'
    '        dest:\n'
    '          index: {3}\n'
    '    filters:\n'
    '    - filtertype: pattern\n'
    '      kind: prefix\n'
    '      value: notfound\n'
)

remote_reindex = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Reindex from remote"\n'
    '    action: reindex\n'
    '    options:\n'
    '      wait_interval: {0}\n'
    '      max_wait: {1}\n'
    '      request_body:\n'
    '        source:\n'
    '          remote:\n'
    '            host: {2}\n'
    '          index: {3}\n'
    '        dest:\n'
    '          index: {4}\n'
    '      remote_filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: {5}\n'
    '    filters:\n'
    '    - filtertype: none\n'
)
migration_reindex = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Reindex from remote"\n'
    '    action: reindex\n'
    '    options:\n'
    '      wait_interval: {0}\n'
    '      max_wait: {1}\n'
    '      migration_prefix: {2}\n'
    '      migration_suffix: {3}\n'
    '      request_body:\n'
    '        source:\n'
    '          remote:\n'
    '            host: {4}\n'
    '          index: {5}\n'
    '        dest:\n'
    '          index: {6}\n'
    '      remote_filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: {7}\n'
    '    filters:\n'
    '    - filtertype: none\n'
)
test_945 = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    action: delete_indices\n'
    '    description: >-\n'
    '      Delete indices older than 7 days\n'
    '    options:\n'
    '      continue_if_exception: False\n'
    '      disable_action: False\n'
    '    filters:\n'
    '    - filtertype: pattern\n'
    '      kind: prefix\n'
    '      value: logstash-\n'
    '      exclude:\n'
    '      - filtertype: age\n'
    '        source: name\n'
    '        direction: older\n'
    '        unit: days\n'
    '        unit_count: 7\n'
)
index_settings = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Act on indices as filtered"\n'
    '    action: index_settings\n'
    '    options:\n'
    '      index_settings:\n'
    '        index:\n'
    '          {0}: {1}\n'
    '      ignore_unavailable: {2}\n'
    '      preserve_existing: {3}\n'
    '    filters:\n'
    '      - filtertype: pattern\n'
    '        kind: prefix\n'
    '        value: my\n'
)

ilm_delete_proto = (
    '---\n'
    'actions:\n'
    '  1:\n'
    '    description: "Delete indices as filtered"\n'
    '    action: delete_indices\n'
    '    options:\n'
    '      allow_ilm_indices: {9}\n'
    '    filters:\n'
    '      - filtertype: {0}\n'
    '        source: {1}\n'
    '        direction: {2}\n'
    '        timestring: {3}\n'
    '        unit: {4}\n'
    '        unit_count: {5}\n'
    '        field: {6}\n'
    '        stats_result: {7}\n'
    '        epoch: {8}\n'
)



"""test_action_indexsettings"""
# pylint: disable=missing-function-docstring, missing-class-docstring, protected-access, attribute-defined-outside-init
from unittest import TestCase
from unittest.mock import Mock
from curator.actions import IndexSettings
from curator.exceptions import ActionError, ConfigurationError, MissingArgument
from curator import IndexList
# Get test variables and constants from a single source
from . import testvars

class TestActionIndexSettings(TestCase):
    VERSION = {'version': {'number': '8.0.0'} }
    def builder(self):
        self.client = Mock()
        self.client.info.return_value = self.VERSION
        self.client.cat.indices.return_value = testvars.state_one
        self.client.indices.get_settings.return_value = testvars.settings_one
        self.client.indices.stats.return_value = testvars.stats_one
        self.client.indices.exists_alias.return_value = False
        self.ilo = IndexList(self.client)
    def test_init_raise_bad_index_list(self):
        self.assertRaises(TypeError, IndexSettings, 'invalid')
    def test_init_no_index_settings(self):
        self.builder()
        _ = IndexSettings(self.ilo, {'index':{'refresh_interval':'1s'}})
        self.assertRaises(MissingArgument, IndexSettings, self.ilo, {})
    def test_init_bad_index_settings(self):
        self.builder()
        _ = IndexSettings(self.ilo, {'index':{'refresh_interval':'1s'}})
        self.assertRaises(ConfigurationError, IndexSettings, self.ilo, {'a':'b'})
    def test_init(self):
        self.builder()
        iso = IndexSettings(self.ilo, {'index':{'refresh_interval':'1s'}})
        self.assertEqual(self.ilo, iso.index_list)
        self.assertEqual(self.client, iso.client)
    def test_static_settings(self):
        static = [
            'number_of_shards',
            'shard',
            'codec',
            'routing_partition_size',
        ]
        self.builder()
        iso = IndexSettings(self.ilo, {'index':{'refresh_interval':'1s'}})
        self.assertEqual(static, iso._static_settings())
    def test_dynamic_settings(self):
        self.builder()
        dynamic = [
            'number_of_replicas',
            'auto_expand_replicas',
            'refresh_interval',
            'max_result_window',
            'max_rescore_window',
            'blocks',
            'max_refresh_listeners',
            'mapping',
            'merge',
            'translog',
        ]
        iso = IndexSettings(self.ilo, {'index':{'refresh_interval':'1s'}})
        self.assertEqual(dynamic, iso._dynamic_settings())
    def test_settings_check_raises_with_opened(self):
        self.builder()
        self.ilo.get_index_state()
        self.ilo.get_index_settings()
        iso = IndexSettings(self.ilo, {'index':{'codec':'best_compression'}})
        self.assertRaises(ActionError, iso._settings_check)
    def test_settings_check_no_raise_with_ignore_unavailable(self):
        self.builder()
        iso = IndexSettings(
            self.ilo, {'index':{'codec':'best_compression'}}, ignore_unavailable=True
        )
        self.assertIsNone(iso._settings_check())
    def test_settings_check_no_raise_with_dynamic_settings(self):
        self.builder()
        iso = IndexSettings(self.ilo, {'index':{'refresh_interval':'1s'}})
        self.assertIsNone(iso._settings_check())
    def test_settings_check_no_raise_with_unknown(self):
        self.builder()
        iso = IndexSettings(self.ilo, {'index':{'foobar':'1s'}})
        self.assertIsNone(iso._settings_check())
    def test_settings_dry_run(self):
        self.builder()
        iso = IndexSettings(self.ilo, {'index':{'refresh_interval':'1s'}})
        self.assertIsNone(iso.do_dry_run())
    def test_settings_do_action(self):
        self.builder()
        self.client.indices.put_settings.return_value = {"acknowledged":True}
        iso = IndexSettings(self.ilo, {'index':{'refresh_interval':'1s'}})
        self.assertIsNone(iso.do_action())
    def test_settings_do_action_raises(self):
        self.builder()
        self.client.indices.put_settings.side_effect = testvars.fake_fail
        iso = IndexSettings(self.ilo, {'index':{'refresh_interval':'1s'}})
        self.assertRaises(Exception, iso.do_action)



"""es_repo_mgr CLI"""

import sys
import logging
import pprint
import click
from elasticsearch9 import ApiError, NotFoundError
from es_client.defaults import LOGGING_SETTINGS, SHOW_OPTION
from es_client.builder import Builder
from es_client.config import (
    cli_opts,
    context_settings,
    generate_configdict,
    get_config,
)
from es_client.logging import configure_logging
from es_client.utils import option_wrapper
from curator.defaults.settings import (
    CLICK_DRYRUN,
    VERSION_MAX,
    VERSION_MIN,
    default_config_file,
    footer,
)
from curator.helpers.getters import get_repository
from curator._version import __version__

ONOFF = {'on': '', 'off': 'no-'}
click_opt_wrap = option_wrapper()


# pylint: disable=unused-argument
def delete_callback(ctx, param, value):
    """Callback if command ``delete`` called

    If the action is ``delete``, this is the :py:class:`click.Parameter` callback
    function if you used the ``--yes`` flag.

    :param ctx: The Click Context
    :param param: The parameter name
    :param value: The value

    :type ctx: :py:class:`~.click.Context`
    :type param: str
    :type value: str
    """
    # It complains if ``param`` isn't passed as a positional parameter, but this
    # is a one-trick pony callback. We don't need to know that you selected
    # ``--yes`` as the parameter.
    if not value:
        ctx.abort()


def show_repos(client):
    """Show all repositories

    :param client: A client connection object
    :type client: :py:class:`~.elasticsearch.Elasticsearch`
    :rtype: None
    """
    logger = logging.getLogger(__name__)
    for repository in sorted(get_repository(client, '*').keys()):
        logger.debug('REPOSITORY = %s', repository)
        click.echo(f'{repository}')
    sys.exit(0)


def get_client(ctx):
    """
    :param ctx: The :py:class:`click` Context
    :type ctx: :py:class:`~.click.Context`

    :returns: A client connection object
    :rtype: :py:class:`~.elasticsearch.Elasticsearch`
    """
    builder = Builder(
        configdict=ctx.obj['configdict'],
        version_max=VERSION_MAX,
        version_min=VERSION_MIN,
    )
    try:
        builder.connect()
    # pylint: disable=broad-except
    except Exception as exc:
        click.echo(f'Exception encountered: {exc}')
        sys.exit(1)
    return builder.client


# pylint: disable=broad-except
def create_repo(ctx, repo_name=None, repo_type=None, repo_settings=None, verify=False):
    """
    Call :py:meth:`~.elasticsearch.client.SnapshotClient.create_repository` to
    create a snapshot repository from the provided arguments

    :param ctx: The :py:class:`click` Context
    :param repo_name: The repository name
    :param repo_type: The repository name
    :param repo_settings: Settings to configure the repository
    :param verify: Whether to verify repository access

    :type ctx: :py:class:`~.click.Context`
    :type repo_name: str
    :type repo_type: str
    :type repo_settings: dict
    :type verify: bool
    :rtype: None
    """
    logger = logging.getLogger('curator.repomgrcli.create_repo')
    client = get_client(ctx)
    request_body = {'type': repo_type, 'settings': repo_settings}
    try:
        client.snapshot.create_repository(
            name=repo_name, body=request_body, verify=verify
        )
    except ApiError as exc:
        if exc.meta.status >= 500:
            logger.critical('Server-side error!')
            if exc.body['error']['type'] == 'repository_exception':
                logger.critical('Repository exception: %s', exc.body)
        click.echo(f'Repository exception: {exc.body}')
        sys.exit(1)
    except Exception as exc:
        logger.critical('Encountered exception %s', exc)
        sys.exit(1)


# pylint: disable=unused-argument, invalid-name, line-too-long
@click.command(short_help='Azure Repository')
@click.option('--name', required=True, type=str, help='Repository name')
@click.option(
    '--client',
    default='default',
    show_default=True,
    type=str,
    help='Azure named client to use.',
)
@click.option(
    '--container',
    default='elasticsearch-snapshots',
    show_default=True,
    type=str,
    help=(
        'Container name. You must create the Azure container before creating '
        'the repository.'
    ),
)
@click.option(
    '--base_path',
    default='',
    show_default=True,
    type=str,
    help=(
        'Specifies the path within container to repository data. Defaults to '
        'empty (root directory).'
    ),
)
@click.option(
    '--chunk_size', type=str, help='Chunk size, e.g. 1g, 10m, 5k. [unbounded]'
)
@click.option(
    '--compress/--no-compress',
    default=True,
    show_default=True,
    help='Enable/Disable metadata compression.',
)
@click.option(
    '--max_restore_rate',
    type=str,
    default='20mb',
    show_default=True,
    help='Throttles per node restore rate (per second).',
)
@click.option(
    '--max_snapshot_rate',
    type=str,
    default='20mb',
    show_default=True,
    help='Throttles per node snapshot rate (per second).',
)
@click.option('--readonly', is_flag=True, help='If set, the repository is read-only.')
@click.option(
    '--location_mode',
    default='primary_only',
    type=click.Choice(['primary_only', 'secondary_only']),
    help='Note that if you set it to secondary_only, it will force readonly to true.',
)
@click.option('--verify', is_flag=True, help='Verify repository after creation.')
@click.pass_context
def azure(
    ctx,
    name,
    client,
    container,
    base_path,
    chunk_size,
    compress,
    max_restore_rate,
    max_snapshot_rate,
    readonly,
    location_mode,
    verify,
):
    """Create an Azure repository."""
    azure_settings = {
        'client': client,
        'container': container,
        'base_path': base_path,
        'chunk_size': chunk_size,
        'compress': compress,
        'max_restore_bytes_per_sec': max_restore_rate,
        'max_snapshot_bytes_per_sec': max_snapshot_rate,
        'readonly': readonly,
        'location_mode': location_mode,
    }
    create_repo(
        ctx,
        repo_name=name,
        repo_type='azure',
        repo_settings=azure_settings,
        verify=verify,
    )


# pylint: disable=unused-argument, invalid-name, line-too-long
@click.command(short_help='Google Cloud Storage Repository')
@click.option('--name', required=True, type=str, help='Repository name')
@click.option(
    '--bucket',
    required=True,
    type=str,
    help='The name of the bucket to be used for snapshots.',
)
@click.option(
    '--client',
    default='default',
    show_default=True,
    type=str,
    help='The name of the client to use to connect to Google Cloud Storage.',
)
@click.option(
    '--base_path',
    default='',
    show_default=True,
    type=str,
    help=(
        'Specifies the path within bucket to repository data. '
        'Defaults to the root of the bucket.'
    ),
)
@click.option(
    '--chunk_size', type=str, help='Chunk size, e.g. 1g, 10m, 5k. [unbounded]'
)
@click.option(
    '--compress/--no-compress',
    default=True,
    show_default=True,
    help='Enable/Disable metadata compression.',
)
@click.option(
    '--max_restore_rate',
    type=str,
    default='20mb',
    show_default=True,
    help='Throttles per node restore rate (per second).',
)
@click.option(
    '--max_snapshot_rate',
    type=str,
    default='20mb',
    show_default=True,
    help='Throttles per node snapshot rate (per second).',
)
@click.option('--readonly', is_flag=True, help='If set, the repository is read-only.')
@click.option('--verify', is_flag=True, help='Verify repository after creation.')
@click.pass_context
def gcs(
    ctx,
    name,
    bucket,
    client,
    base_path,
    chunk_size,
    compress,
    max_restore_rate,
    max_snapshot_rate,
    readonly,
    verify,
):
    """Create a Google Cloud Storage repository."""
    gcs_settings = {
        'bucket': bucket,
        'client': client,
        'base_path': base_path,
        'chunk_size': chunk_size,
        'compress': compress,
        'max_restore_bytes_per_sec': max_restore_rate,
        'max_snapshot_bytes_per_sec': max_snapshot_rate,
        'readonly': readonly,
    }
    create_repo(
        ctx, repo_name=name, repo_type='gcs', repo_settings=gcs_settings, verify=verify
    )


# pylint: disable=unused-argument, invalid-name, line-too-long
@click.command(short_help='S3 Repository')
@click.option('--name', required=True, type=str, help='Repository name')
@click.option(
    '--bucket',
    required=True,
    type=str,
    help='The bucket name must adhere to Amazon’s S3 bucket naming rules.',
)
@click.option(
    '--client',
    default='default',
    show_default=True,
    type=str,
    help='The name of the S3 client to use to connect to S3.',
)
@click.option(
    '--base_path',
    default='',
    show_default=True,
    type=str,
    help=(
        'Specifies the path to the repository data within its bucket. Defaults '
        'to an empty string, meaning that the repository is at the root of the '
        'bucket. The value of this setting should not start or end with a /.'
    ),
)
@click.option(
    '--chunk_size', type=str, help='Chunk size, e.g. 1g, 10m, 5k. [unbounded]'
)
@click.option(
    '--compress/--no-compress',
    default=True,
    show_default=True,
    help='Enable/Disable metadata compression.',
)
@click.option(
    '--max_restore_rate',
    type=str,
    default='20mb',
    show_default=True,
    help='Throttles per node restore rate (per second).',
)
@click.option(
    '--max_snapshot_rate',
    type=str,
    default='20mb',
    show_default=True,
    help='Throttles per node snapshot rate (per second).',
)
@click.option('--readonly', is_flag=True, help='If set, the repository is read-only.')
@click.option(
    '--server_side_encryption',
    is_flag=True,
    help='If set, files are encrypted on server side using AES256 algorithm.',
)
@click.option(
    '--buffer_size',
    default='',
    type=str,
    help=(
        'Minimum threshold below which the chunk is uploaded using a single '
        'request. Must be between 5mb and 5gb.'
    ),
)
@click.option(
    '--canned_acl',
    default='private',
    type=click.Choice(
        [
            'private',
            'public-read',
            'public-read-write',
            'authenticated-read',
            'log-delivery-write',
            'bucket-owner-read',
            'bucket-owner-full-control',
        ]
    ),
    help=(
        'When the S3 repository creates buckets and objects, it adds the canned '
        'ACL into the buckets and objects.'
    ),
)
@click.option(
    '--storage_class',
    default='standard',
    type=click.Choice(
        [
            'standard',
            'reduced_redundancy',
            'standard_ia',
            'onezone_ia',
            'intelligent_tiering',
        ]
    ),
    help='Sets the S3 storage class for objects stored in the snapshot repository.',
)
@click.option('--verify', is_flag=True, help='Verify repository after creation.')
@click.pass_context
def s3(
    ctx,
    name,
    bucket,
    client,
    base_path,
    chunk_size,
    compress,
    max_restore_rate,
    max_snapshot_rate,
    readonly,
    server_side_encryption,
    buffer_size,
    canned_acl,
    storage_class,
    verify,
):
    """
    Create an S3 repository.
    """
    s3_settings = {
        'bucket': bucket,
        'client': client,
        'base_path': base_path,
        'chunk_size': chunk_size,
        'compress': compress,
        'max_restore_bytes_per_sec': max_restore_rate,
        'max_snapshot_bytes_per_sec': max_snapshot_rate,
        'readonly': readonly,
        'server_side_encryption': server_side_encryption,
        'buffer_size': buffer_size,
        'canned_acl': canned_acl,
        'storage_class': storage_class,
    }
    create_repo(
        ctx, repo_name=name, repo_type='s3', repo_settings=s3_settings, verify=verify
    )


# pylint: disable=unused-argument, invalid-name
@click.command(short_help='Shared Filesystem Repository')
@click.option('--name', required=True, type=str, help='Repository name')
@click.option(
    '--location',
    required=True,
    type=str,
    help=(
        'Shared file-system location. Must match remote path, & be accessible '
        'to all master & data nodes'
    ),
)
@click.option(
    '--compress/--no-compress',
    default=True,
    show_default=True,
    help='Enable/Disable metadata compression.',
)
@click.option(
    '--chunk_size', type=str, help='Chunk size, e.g. 1g, 10m, 5k. [unbounded]'
)
@click.option(
    '--max_snapshots',
    default=2147483647,
    type=int,
    help=(
        'Maximum number of snapshots the repository can contain. '
        'Defaults to Integer.MAX_VALUE, which is 2147483647.'
    ),
)
@click.option(
    '--max_restore_rate',
    type=str,
    default='20mb',
    show_default=True,
    help='Throttles per node restore rate (per second).',
)
@click.option(
    '--max_snapshot_rate',
    type=str,
    default='20mb',
    show_default=True,
    help='Throttles per node snapshot rate (per second).',
)
@click.option('--readonly', is_flag=True, help='If set, the repository is read-only.')
@click.option('--verify', is_flag=True, help='Verify repository after creation.')
@click.pass_context
def fs(
    ctx,
    name,
    location,
    compress,
    chunk_size,
    max_snapshots,
    max_restore_rate,
    max_snapshot_rate,
    readonly,
    verify,
):
    """
    Create a filesystem repository.
    """
    fs_settings = {
        'location': location,
        'compress': compress,
        'chunk_size': chunk_size,
        'max_number_of_snapshots': max_snapshots,
        'max_restore_bytes_per_sec': max_restore_rate,
        'max_snapshot_bytes_per_sec': max_snapshot_rate,
        'readonly': readonly,
    }
    create_repo(
        ctx, repo_name=name, repo_type='fs', repo_settings=fs_settings, verify=verify
    )


# pylint: disable=unused-argument, invalid-name
@click.command(short_help='Read-only URL Repository')
@click.option('--name', required=True, type=str, help='Repository name')
@click.option(
    '--chunk_size', type=str, help='Chunk size, e.g. 1g, 10m, 5k. [unbounded]'
)
@click.option(
    '--http_max_retries',
    type=int,
    default=5,
    show_default=True,
    help='Maximum number of retries for http and https',
)
@click.option(
    '--http_socket_timeout',
    type=str,
    default='50s',
    show_default=True,
    help='Maximum wait time for data transfers over a connection.',
)
@click.option(
    '--compress/--no-compress',
    default=True,
    show_default=True,
    help='Enable/Disable metadata compression.',
)
@click.option(
    '--max_snapshots',
    default=2147483647,
    type=int,
    help=(
        'Maximum number of snapshots the repository can contain. '
        'Defaults to Integer.MAX_VALUE, which is 2147483647.'
    ),
)
@click.option(
    '--max_restore_rate',
    type=str,
    default='20mb',
    show_default=True,
    help='Throttles per node restore rate (per second).',
)
@click.option(
    '--shared_filesystem_url',
    required=True,
    type=str,
    help='URL location of the root of the shared filesystem repository.',
)
@click.option('--verify', is_flag=True, help='Verify repository after creation.')
@click.pass_context
def url(
    ctx,
    name,
    chunk_size,
    http_max_retries,
    http_socket_timeout,
    compress,
    max_snapshots,
    max_restore_rate,
    shared_filesystem_url,
    verify,
):
    """
    Create a read-only url repository.
    """
    url_settings = {
        'chunk_size': chunk_size,
        'http_max_retries': http_max_retries,
        'http_socket_timeout': http_socket_timeout,
        'compress': compress,
        'max_number_of_snapshots': max_snapshots,
        'max_restore_bytes_per_sec': max_restore_rate,
        'url': shared_filesystem_url,
    }
    create_repo(
        ctx, repo_name=name, repo_type='url', repo_settings=url_settings, verify=verify
    )


# pylint: disable=unused-argument, invalid-name
@click.command(short_help='Source-Only Repository')
@click.option('--name', required=True, type=str, help='Repository name')
@click.option(
    '--delegate_type',
    required=True,
    type=click.Choice(['azure', 'gcs', 's3', 'fs']),
    help='Delegated repository type.',
)
@click.option(
    '--location',
    required=True,
    type=str,
    help='For Source-Only, specify the delegated repository name here',
)
@click.option(
    '--compress/--no-compress',
    default=True,
    show_default=True,
    help='Enable/Disable metadata compression.',
)
@click.option(
    '--chunk_size', type=str, help='Chunk size, e.g. 1g, 10m, 5k. [unbounded]'
)
@click.option(
    '--max_snapshots',
    default=2147483647,
    type=int,
    help=(
        'Maximum number of snapshots the repository can contain. '
        'Defaults to Integer.MAX_VALUE, which is 2147483647.'
    ),
)
@click.option(
    '--max_restore_rate',
    type=str,
    default='20mb',
    show_default=True,
    help='Throttles per node restore rate (per second).',
)
@click.option(
    '--max_snapshot_rate',
    type=str,
    default='20mb',
    show_default=True,
    help='Throttles per node snapshot rate (per second).',
)
@click.option('--readonly', is_flag=True, help='If set, the repository is read-only.')
@click.option('--verify', is_flag=True, help='Verify repository after creation.')
@click.pass_context
def source(
    ctx,
    name,
    delegate_type,
    location,
    compress,
    chunk_size,
    max_snapshots,
    max_restore_rate,
    max_snapshot_rate,
    readonly,
    verify,
):
    """
    Create a filesystem repository.
    """
    source_settings = {
        'chunk_size': chunk_size,
        'compress': compress,
        'delegate_type': delegate_type,
        'location': location,
        'max_number_of_snapshots': max_snapshots,
        'max_restore_bytes_per_sec': max_restore_rate,
        'max_snapshot_bytes_per_sec': max_snapshot_rate,
        'readonly': readonly,
    }
    create_repo(
        ctx,
        repo_name=name,
        repo_type='source',
        repo_settings=source_settings,
        verify=verify,
    )


# pylint: disable=R0913, R0914, W0622
@click.group(
    'repo_mgr_cli', context_settings=context_settings(), epilog=footer(__version__)
)
@click_opt_wrap(*cli_opts('config'))
@click_opt_wrap(*cli_opts('hosts'))
@click_opt_wrap(*cli_opts('cloud_id'))
@click_opt_wrap(*cli_opts('api_token'))
@click_opt_wrap(*cli_opts('id'))
@click_opt_wrap(*cli_opts('api_key'))
@click_opt_wrap(*cli_opts('username'))
@click_opt_wrap(*cli_opts('password'))
@click_opt_wrap(*cli_opts('bearer_auth'))
@click_opt_wrap(*cli_opts('opaque_id'))
@click_opt_wrap(*cli_opts('request_timeout'))
@click_opt_wrap(*cli_opts('http_compress', onoff=ONOFF))
@click_opt_wrap(*cli_opts('verify_certs', onoff=ONOFF))
@click_opt_wrap(*cli_opts('ca_certs'))
@click_opt_wrap(*cli_opts('client_cert'))
@click_opt_wrap(*cli_opts('client_key'))
@click_opt_wrap(*cli_opts('ssl_assert_hostname'))
@click_opt_wrap(*cli_opts('ssl_assert_fingerprint'))
@click_opt_wrap(*cli_opts('ssl_version'))
@click_opt_wrap(*cli_opts('master-only', onoff=ONOFF))
@click_opt_wrap(*cli_opts('skip_version_test', onoff=ONOFF))
@click_opt_wrap(*cli_opts('dry-run', settings=CLICK_DRYRUN))
@click_opt_wrap(*cli_opts('loglevel', settings=LOGGING_SETTINGS))
@click_opt_wrap(*cli_opts('logfile', settings=LOGGING_SETTINGS))
@click_opt_wrap(*cli_opts('logformat', settings=LOGGING_SETTINGS))
@click.version_option(__version__, '-v', '--version', prog_name="es_repo_mgr")
@click.pass_context
def repo_mgr_cli(
    ctx,
    config,
    hosts,
    cloud_id,
    api_token,
    id,
    api_key,
    username,
    password,
    bearer_auth,
    opaque_id,
    request_timeout,
    http_compress,
    verify_certs,
    ca_certs,
    client_cert,
    client_key,
    ssl_assert_hostname,
    ssl_assert_fingerprint,
    ssl_version,
    master_only,
    skip_version_test,
    dry_run,
    loglevel,
    logfile,
    logformat,
):
    """
    Repository manager for Elasticsearch Curator

    The default $HOME/.curator/curator.yml configuration file (--config)
    can be used but is not needed.

    Command-line settings will always override YAML configuration settings.

    Some less-frequently used client configuration options are now hidden.
    To see the full list, run:

        es_repo_mgr show-all-options
    """
    ctx.obj = {}
    ctx.obj['dry_run'] = dry_run
    ctx.obj['default_config'] = default_config_file()
    get_config(ctx)
    configure_logging(ctx)
    logger = logging.getLogger('curator.repomgrcli')
    generate_configdict(ctx)
    logger.debug('Exiting initial command function...')


@repo_mgr_cli.command(
    'show-all-options',
    context_settings=context_settings(),
    short_help='Show all configuration options',
    epilog=footer(__version__),
)
@click_opt_wrap(*cli_opts('config'))
@click_opt_wrap(*cli_opts('hosts'))
@click_opt_wrap(*cli_opts('cloud_id'))
@click_opt_wrap(*cli_opts('api_token'))
@click_opt_wrap(*cli_opts('id'))
@click_opt_wrap(*cli_opts('api_key'))
@click_opt_wrap(*cli_opts('username'))
@click_opt_wrap(*cli_opts('password'))
@click_opt_wrap(*cli_opts('bearer_auth', override=SHOW_OPTION))
@click_opt_wrap(*cli_opts('opaque_id', override=SHOW_OPTION))
@click_opt_wrap(*cli_opts('request_timeout'))
@click_opt_wrap(*cli_opts('http_compress', onoff=ONOFF, override=SHOW_OPTION))
@click_opt_wrap(*cli_opts('verify_certs', onoff=ONOFF))
@click_opt_wrap(*cli_opts('ca_certs'))
@click_opt_wrap(*cli_opts('client_cert'))
@click_opt_wrap(*cli_opts('client_key'))
@click_opt_wrap(*cli_opts('ssl_assert_hostname', override=SHOW_OPTION))
@click_opt_wrap(*cli_opts('ssl_assert_fingerprint', override=SHOW_OPTION))
@click_opt_wrap(*cli_opts('ssl_version', override=SHOW_OPTION))
@click_opt_wrap(*cli_opts('master-only', onoff=ONOFF, override=SHOW_OPTION))
@click_opt_wrap(*cli_opts('skip_version_test', onoff=ONOFF, override=SHOW_OPTION))
@click_opt_wrap(*cli_opts('dry-run', settings=CLICK_DRYRUN))
@click_opt_wrap(*cli_opts('loglevel', settings=LOGGING_SETTINGS))
@click_opt_wrap(*cli_opts('logfile', settings=LOGGING_SETTINGS))
@click_opt_wrap(*cli_opts('logformat', settings=LOGGING_SETTINGS))
@click.version_option(__version__, '-v', '--version', prog_name="es_repo_mgr")
@click.pass_context
def show_all_options(
    ctx,
    config,
    hosts,
    cloud_id,
    api_token,
    id,
    api_key,
    username,
    password,
    bearer_auth,
    opaque_id,
    request_timeout,
    http_compress,
    verify_certs,
    ca_certs,
    client_cert,
    client_key,
    ssl_assert_hostname,
    ssl_assert_fingerprint,
    ssl_version,
    master_only,
    skip_version_test,
    dry_run,
    loglevel,
    logfile,
    logformat,
):
    """
    ALL CLIENT OPTIONS

    The following is the full list of settings available for configuring a
    connection using command-line options.
    """
    ctx = click.get_current_context()
    click.echo(ctx.get_help())
    ctx.exit()


@repo_mgr_cli.group('create')
@click.pass_context
def _create(ctx):
    """Create an Elasticsearch repository"""


_create.add_command(azure)
_create.add_command(gcs)
_create.add_command(s3)
_create.add_command(fs)
_create.add_command(url)
_create.add_command(source)


@repo_mgr_cli.command('show')
@click.pass_context
def show(ctx):
    """Show all repositories"""
    client = get_client(ctx)
    show_repos(client)


@repo_mgr_cli.command('delete')
@click.option('--name', required=True, help='Repository name', type=str)
@click.option(
    '--yes',
    is_flag=True,
    callback=delete_callback,
    expose_value=False,
    prompt='Are you sure you want to delete the repository?',
)
@click.pass_context
def _delete(ctx, name):
    """
    Delete an Elasticsearch repository
    """
    logger = logging.getLogger('curator.repomgrcli._delete')
    client = get_client(ctx)
    try:
        logger.info('Deleting repository %s...', name)
        client.snapshot.delete_repository(name=name)
    except NotFoundError:
        logger.error('Unable to delete repository: %s  Not Found.', name)
        sys.exit(1)


@repo_mgr_cli.command('info')
@click.pass_context
def info(ctx):
    """Show cluster info"""
    client = get_client(ctx)
    pp = pprint.PrettyPrinter(indent=4)
    output = dict(client.info())
    click.echo(f'{pp.pprint(output)}')



"""Alias unit tests"""
# pylint: disable=missing-function-docstring, missing-class-docstring, invalid-name, line-too-long, attribute-defined-outside-init
from unittest import TestCase
from unittest.mock import Mock
from curator import IndexList
from curator.exceptions import ActionError, FailedExecution, MissingArgument, NoIndices
from curator.actions.alias import Alias
from . import testvars

class TestActionAlias(TestCase):
    VERSION = {'version': {'number': '8.0.0'} }
    def builder(self):
        self.client = Mock()
        self.client.info.return_value = self.VERSION
        self.client.cat.indices.return_value = testvars.state_one
        self.client.indices.get_settings.return_value = testvars.settings_one
        self.client.indices.stats.return_value = testvars.stats_one
        self.client.indices.exists_alias.return_value = False
        self.ilo = IndexList(self.client)
    def builder2(self):
        self.client = Mock()
        self.client.info.return_value = self.VERSION
        self.client.cat.indices.return_value = testvars.state_two
        self.client.indices.get_settings.return_value = testvars.settings_two
        self.client.indices.stats.return_value = testvars.stats_two
        self.client.indices.exists_alias.return_value = False
        self.ilo = IndexList(self.client)
    def test_init_raise(self):
        self.assertRaises(MissingArgument, Alias)
    def test_add_raises_on_missing_parameter(self):
        self.builder()
        ao = Alias(name='alias')
        self.assertRaises(TypeError, ao.add)
    def test_add_raises_on_invalid_parameter(self):
        self.builder()
        ao = Alias(name='alias')
        self.assertRaises(TypeError, ao.add, [])
    def test_add_single(self):
        self.builder()
        ao = Alias(name='alias')
        ao.add(self.ilo)
        self.assertEqual(testvars.alias_one_add, ao.actions)
    def test_add_single_with_extra_settings(self):
        self.builder()
        esd = {
            'filter' : { 'term' : { 'user' : 'kimchy' } }
        }
        ao = Alias(name='alias', extra_settings=esd)
        ao.add(self.ilo)
        self.assertEqual(testvars.alias_one_add_with_extras, ao.actions)
    def test_remove_single(self):
        self.builder()
        self.client.indices.get_alias.return_value = testvars.settings_1_get_aliases
        ao = Alias(name='my_alias')
        ao.remove(self.ilo)
        self.assertEqual(testvars.alias_one_rm, ao.actions)
    def test_add_multiple(self):
        self.builder2()
        ao = Alias(name='alias')
        ao.add(self.ilo)
        cmp = sorted(ao.actions, key=lambda k: k['add']['index'])
        self.assertEqual(testvars.alias_two_add, cmp)
    def test_remove_multiple(self):
        self.builder2()
        self.client.indices.get_alias.return_value = testvars.settings_2_get_aliases
        ao = Alias(name='my_alias')
        ao.remove(self.ilo)
        cmp = sorted(ao.actions, key=lambda k: k['remove']['index'])
        self.assertEqual(testvars.alias_two_rm, cmp)
    def test_raise_action_error_on_empty_body(self):
        self.builder()
        ao = Alias(name='alias')
        self.assertRaises(ActionError, ao.check_actions)
    def test_raise_no_indices_on_empty_body_when_warn_if_no_indices(self):
        self.builder()
        # empty it, so there can be no body
        self.ilo.indices = []
        ao = Alias(name='alias')
        ao.add(self.ilo, warn_if_no_indices=True)
        self.assertRaises(NoIndices, ao.check_actions)
    def test_do_dry_run(self):
        self.builder()
        self.client.indices.update_aliases.return_value = testvars.alias_success
        ao = Alias(name='alias')
        ao.add(self.ilo)
        self.assertIsNone(ao.do_dry_run())
    def test_do_action(self):
        self.builder()
        self.client.indices.update_aliases.return_value = testvars.alias_success
        ao = Alias(name='alias')
        ao.add(self.ilo)
        self.assertIsNone(ao.do_action())
    def test_do_action_raises_exception(self):
        self.builder()
        self.client.indices.update_aliases.return_value = testvars.alias_success
        self.client.indices.update_aliases.side_effect = testvars.four_oh_one
        ao = Alias(name='alias')
        ao.add(self.ilo)
        self.assertRaises(FailedExecution, ao.do_action)



"""Alias unit tests"""
# pylint: disable=missing-function-docstring, missing-class-docstring, invalid-name, line-too-long, attribute-defined-outside-init
from unittest import TestCase
from unittest.mock import Mock
from curator import IndexList
from curator.exceptions import FailedExecution
from curator.actions.close import Close
from . import testvars

class TestActionClose(TestCase):
    VERSION = {'version': {'number': '5.0.0'} }
    def builder(self):
        self.client = Mock()
        self.client.info.return_value = self.VERSION
        self.client.cat.indices.return_value = testvars.state_one
        self.client.indices.get_settings.return_value = testvars.settings_one
        self.client.indices.stats.return_value = testvars.stats_one
        self.client.indices.flush_synced.return_value = testvars.synced_pass
        self.client.indices.exists_alias.return_value = False
        self.client.indices.close.return_value = None
        self.ilo = IndexList(self.client)
    def test_init_raise(self):
        self.assertRaises(TypeError, Close, 'invalid')
    def test_init(self):
        self.builder()
        self.client.indices.flush_synced.return_value = None
        self.client.indices.close.return_value = None
        clo = Close(self.ilo)
        self.assertEqual(self.ilo, clo.index_list)
        self.assertEqual(self.client, clo.client)
    def test_do_dry_run(self):
        self.builder()
        self.ilo = IndexList(self.client)
        clo = Close(self.ilo)
        self.assertIsNone(clo.do_dry_run())
    def test_do_action(self):
        self.builder()
        self.ilo = IndexList(self.client)
        clo = Close(self.ilo)
        self.assertIsNone(clo.do_action())
    def test_do_action_with_delete_aliases(self):
        self.builder()
        self.ilo = IndexList(self.client)
        clo = Close(self.ilo, delete_aliases=True)
        self.assertIsNone(clo.do_action())
    def test_do_action_with_skip_flush(self):
        self.builder()
        self.ilo = IndexList(self.client)
        clo = Close(self.ilo, skip_flush=True)
        self.assertIsNone(clo.do_action())
    def test_do_action_raises_exception(self):
        self.builder()
        self.client.indices.close.side_effect = testvars.fake_fail
        self.ilo = IndexList(self.client)
        clo = Close(self.ilo)
        self.assertRaises(FailedExecution, clo.do_action)
    def test_do_action_delete_aliases_with_exception(self):
        self.builder()
        self.ilo = IndexList(self.client)
        self.client.indices.delete_alias.side_effect = testvars.fake_fail
        clo = Close(self.ilo, delete_aliases=True)
        self.assertIsNone(clo.do_action())



"""Unit tests for utils"""

from unittest import TestCase

# import pytest
from unittest.mock import Mock

# from curator.exceptions import MissingArgument
from curator.indexlist import IndexList
from curator.helpers.utils import (
    chunk_index_list,
    show_dry_run,
    to_csv,
    multitarget_fix,
    multitarget_match,
)
from . import testvars

FAKE_FAIL = Exception('Simulated Failure')


class TestShowDryRun(TestCase):
    """TestShowDryRun

    Test helpers.utils.show_dry_run functionality.
    """

    # For now, since it's a pain to capture logging output, this is just a
    # simple code coverage run
    def test_index_list(self):
        """test_index_list

        Should split really long index list (well, really long index names) into
        2 chunks
        """
        client = Mock()
        client.info.return_value = {'version': {'number': '8.0.0'}}
        client.cat.indices.return_value = testvars.state_two
        client.indices.get_settings.return_value = testvars.settings_two
        client.indices.stats.return_value = testvars.stats_two
        client.indices.exists_alias.return_value = False
        client.field_stats.return_value = testvars.fieldstats_two
        ilst = IndexList(client)
        assert None is show_dry_run(ilst, 'test_action')


class TestChunkIndexList(TestCase):
    """TestToCSV

    Test helpers.utils.chunk_index_list functionality.
    """

    def test_big_list(self):
        """test_big_list

        Should split really long index list (well, really long index names) into
        2 chunks
        """
        indices = []
        for i in range(100, 150):
            indices.append(
                (
                    'superlongindexnamebyanystandardyouchoosethis'
                    'issillyhowbigcanthisgetbeforeitbreaks'
                )
                + str(i)
            )
        assert 2 == len(chunk_index_list(indices))

    def test_small_list(self):
        """test_small_list

        Should not split short index list
        """
        assert 1 == len(chunk_index_list(['short', 'list', 'of', 'indices']))


class TestToCSV(TestCase):
    """TestToCSV

    Test helpers.utils.to_csv functionality.
    """

    def test_to_csv_will_return_csv(self):
        """test_to_csv_will_return_csv

        Should return csv version of provided list
        """
        lst = ["a", "b", "c", "d"]
        csv = "a,b,c,d"
        # self.assertEqual(csv, to_csv(lst))
        assert csv == to_csv(lst)

    def test_to_csv_will_return_single(self):
        """test_to_csv_will_return_single

        Should return single string of one-element list, no comma
        """
        lst = ["a"]
        csv = "a"
        # self.assertEqual(csv, to_csv(lst))
        assert csv == to_csv(lst)

    def test_to_csv_will_return_none(self):
        """test_to_csv_will_return_None

        Should return :py:class:`None` if empty list is passed
        """
        lst = []
        # self.assertIsNone(to_csv(lst))
        assert None is to_csv(lst)


class TestMultiTarget(TestCase):
    """TestMultiTarget

    Test helpers.utils.multitarget_* functionality.
    """

    SIMPLE = ['index1', 'index2', 'index3']
    COMPLEX = ['index1', 'index2', 'index3', 'not-index1', 'not-index2', 'not-index3']

    def test_multitarget_fix_no_elements(self):
        """test_multitarget_fix_no_elements

        If there are no elements in the pattern, return a wildcard
        """
        assert '*' == multitarget_fix('')

    def test_multitarget_fix_only_negative(self):
        """test_multitarget_fix_only_negative

        If there are only negative elements in the pattern, return those prepended
        by a wildcard
        """
        assert '*,-index1,-index2' == multitarget_fix('-index1,-index2')

    def test_multitarget_fix_positive_and_negative(self):
        """test_multitarget_fix_positive_and_negative

        If there are both positive and negative elements in the pattern, return the
        original pattern with no added wildcard
        """
        assert 'index1,-index2' == multitarget_fix('index1,-index2')

    def test_multitarget_fix_only_positive(self):
        """test_multitarget_fix_only_positive

        If there are only positive elements in the pattern, return the original
        pattern with no added wildcard
        """
        assert 'index1,index2' == multitarget_fix('index1,index2')

    def test_multitarget_match_no_elements(self):
        """test_multitarget_match_no_elements

        If there are no elements in the pattern, return all indices
        """
        assert self.SIMPLE == multitarget_match('', self.SIMPLE)

    def test_multitarget_match_only_negative(self):
        """test_multitarget_match_only_negative

        Return the indices matching the pattern, which has only negatives
        containing no wildcards
        """
        assert ['index3'] == multitarget_match('-index1,-index2', self.SIMPLE)

    def test_multitarget_match_only_negative_complex(self):
        """test_multitarget_match_only_negative_complex

        Return the indices matching the pattern, which has wildcard negatives
        """
        assert ['index3', 'not-index3'] == multitarget_match('-*1,-*2', self.COMPLEX)

    def test_multitarget_match_positive_and_negative(self):
        """test_multitarget_match_positive_and_negative

        Return the indices matching the pattern, which has both positive and
        negative elements
        """
        assert ['index1'] == multitarget_match('index1,-index2', self.SIMPLE)

    def test_multitarget_match_only_positive(self):
        """test_multitarget_match_only_positive

        Return the indices matching the pattern, which has one positive element
        and no wildcards
        """
        assert ['index2'] == multitarget_match('index2', self.COMPLEX)

    def test_multitarget_match_only_positive_wildcard(self):
        """test_multitarget_match_only_positive_wildcard

        Return the indices matching the pattern, which has one positive element
        that contains a wildcard
        """
        assert ['index2', 'not-index2'] == multitarget_match('*2', self.COMPLEX)



"""Test Allocation action"""

# pylint: disable=C0115, C0116, invalid-name
import os
from . import CuratorTestCase
from . import testvars

HOST = os.environ.get('TEST_ES_SERVER', 'http://127.0.0.1:9200')

TIEREDROUTING = {'allocation': {'include': {'_tier_preference': 'data_content'}}}
KEY = 'tag'
VALUE = 'value'


class TestActionFileAllocation(CuratorTestCase):
    def test_include(self):
        alloc = 'include'
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.allocation_test.format(KEY, VALUE, alloc, False),
        )
        idx1, idx2 = ('my_index', 'not_my_index')
        self.create_index(idx1)
        self.create_index(idx2)
        self.invoke_runner()
        assert (
            VALUE
            == self.client.indices.get_settings(index=idx1)[idx1]['settings']['index'][
                'routing'
            ]['allocation'][alloc][KEY]
        )
        assert (
            TIEREDROUTING
            == self.client.indices.get_settings(index=idx2)[idx2]['settings']['index'][
                'routing'
            ]
        )

    def test_require(self):
        alloc = 'require'
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.allocation_test.format(KEY, VALUE, alloc, False),
        )
        idx1, idx2 = ('my_index', 'not_my_index')
        self.create_index(idx1)
        self.create_index(idx2)
        self.invoke_runner()
        assert (
            VALUE
            == self.client.indices.get_settings(index=idx1)[idx1]['settings']['index'][
                'routing'
            ]['allocation'][alloc][KEY]
        )
        assert (
            TIEREDROUTING
            == self.client.indices.get_settings(index=idx2)[idx2]['settings']['index'][
                'routing'
            ]
        )

    def test_exclude(self):
        alloc = 'exclude'
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.allocation_test.format(KEY, VALUE, alloc, False),
        )
        idx1, idx2 = ('my_index', 'not_my_index')
        self.create_index(idx1)
        self.create_index(idx2)
        self.invoke_runner()
        assert (
            VALUE
            == self.client.indices.get_settings(index=idx1)[idx1]['settings']['index'][
                'routing'
            ]['allocation'][alloc][KEY]
        )
        assert (
            TIEREDROUTING
            == self.client.indices.get_settings(index=idx2)[idx2]['settings']['index'][
                'routing'
            ]
        )

    def test_remove_exclude_with_none_value(self):
        empty = ''  # EMPTYVALUE
        alloc = 'exclude'
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.allocation_test.format(KEY, empty, alloc, False),
        )
        idx1, idx2 = ('my_index', 'not_my_index')
        self.create_index(idx1)
        self.create_index(idx2)
        # Put a setting in place before we start the test.
        self.client.indices.put_settings(
            index=idx1, settings={f'index.routing.allocation.{alloc}.{KEY}': 'bar'}
        )
        # Ensure we _have_ it here first.
        assert (
            'bar'
            == self.client.indices.get_settings(index=idx1)[idx1]['settings']['index'][
                'routing'
            ]['allocation'][alloc][KEY]
        )
        self.invoke_runner()
        assert (
            TIEREDROUTING
            == self.client.indices.get_settings(index=idx1)[idx1]['settings']['index'][
                'routing'
            ]
        )
        assert (
            TIEREDROUTING
            == self.client.indices.get_settings(index=idx2)[idx2]['settings']['index'][
                'routing'
            ]
        )

    def test_invalid_allocation_type(self):
        alloc = 'invalid'
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.allocation_test.format(KEY, VALUE, alloc, False),
        )
        idx1, idx2 = ('my_index', 'not_my_index')
        self.create_index(idx1)
        self.create_index(idx2)
        self.invoke_runner()
        assert 1 == self.result.exit_code

    def test_extra_option(self):
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'], testvars.bad_option_proto_test.format('allocation')
        )
        idx1, idx2 = ('my_index', 'not_my_index')
        self.create_index(idx1)
        self.create_index(idx2)
        self.invoke_runner()
        assert 1 == self.result.exit_code

    def test_skip_closed(self):
        alloc = 'include'
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.allocation_test.format(KEY, VALUE, alloc, False),
        )
        idx1, idx2 = ('my_index', 'not_my_index')
        self.create_index(idx1)
        self.client.indices.close(index=idx1, wait_for_active_shards=0)
        self.create_index(idx2)
        self.invoke_runner()
        assert (
            TIEREDROUTING
            == self.client.indices.get_settings(index=idx1)[idx1]['settings']['index'][
                'routing'
            ]
        )
        assert (
            TIEREDROUTING
            == self.client.indices.get_settings(index=idx2)[idx2]['settings']['index'][
                'routing'
            ]
        )

    def test_wait_for_completion(self):
        alloc = 'require'
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.allocation_test.format(KEY, VALUE, alloc, True),
        )
        idx1, idx2 = ('my_index', 'not_my_index')
        self.create_index(idx1)
        self.create_index(idx2)
        self.invoke_runner()
        assert (
            VALUE
            == self.client.indices.get_settings(index=idx1)[idx1]['settings']['index'][
                'routing'
            ]['allocation'][alloc][KEY]
        )
        assert (
            TIEREDROUTING
            == self.client.indices.get_settings(index=idx2)[idx2]['settings']['index'][
                'routing'
            ]
        )


class TestCLIAllocation(CuratorTestCase):
    def test_include(self):
        alloc = 'include'
        idx1, idx2 = ('my_index', 'not_my_index')
        self.create_index(idx1)
        self.create_index(idx2)
        args = self.get_runner_args()
        args += [
            '--config',
            self.args['configfile'],
            'allocation',
            '--key',
            KEY,
            '--value',
            VALUE,
            '--allocation_type',
            alloc,
            '--wait_for_completion',
            '--filter_list',
            '{"filtertype":"pattern","kind":"prefix","value":"my"}',
        ]
        assert 0 == self.run_subprocess(args, logname='TestCLIAllocation.test_include')
        assert (
            VALUE
            == self.client.indices.get_settings(index=idx1)[idx1]['settings']['index'][
                'routing'
            ]['allocation'][alloc][KEY]
        )
        assert (
            TIEREDROUTING
            == self.client.indices.get_settings(index=idx2)[idx2]['settings']['index'][
                'routing'
            ]
        )



#!/usr/bin/env python
# pylint: disable=broad-except, no-value-for-parameter
"""
Wrapper for running es_repo_mgr from source.

When used with Python 3 Curator requires the locale to be unicode. Any unicode
definitions are acceptable.

To set the locale to be unicode, try:

$ export LC_ALL=en_US.utf8
$ es_repo_mgr [ARGS]

Alternately, you should be able to specify the locale on the command-line:

$ LC_ALL=en_US.utf8 es_repo_mgr [ARGS]

Be sure to substitute your unicode variant for en_US.utf8
"""
import sys
import click
from curator.repomgrcli import repo_mgr_cli

if __name__ == '__main__':
    try:
        repo_mgr_cli()
    except RuntimeError as err:
        click.echo(f'{err}')
        sys.exit(1)
    except Exception as err:
        if 'ASCII' in str(err):
            click.echo(f'{err}')
            click.echo(__doc__)



"""Unit testing for helpers.creators functions"""
from unittest import TestCase
import pytest
from curator.defaults.settings import CURATOR_DOCS, footer
from curator.exceptions import CuratorException
from curator._version import __version__

class TestFooter(TestCase):
    """Test defaults.settings.footer functionality."""
    mytail = 'tail.html'
    ver = __version__.split('.')
    majmin = f'{ver[0]}.{ver[1]}'
    def test_basic_functionality(self):
        """Should return a URL with the major/minor version in the path"""
        expected = f'Learn more at {CURATOR_DOCS}/{self.majmin}/{self.mytail}'
        assert expected == footer(__version__, tail=self.mytail)
    def test_raises_with_nonstring(self):
        """Should raise an exception if a non-string value is passed as version"""
        with pytest.raises(CuratorException):
            footer(1234)
    def test_raises_with_unsplittable_string(self):
        """Should raise an exception if a non-period delimited string value is passed as version"""
        with pytest.raises(CuratorException):
            footer('invalid')



"""Test rollover action functionality"""

# pylint: disable=C0115, C0116, invalid-name
import os
import time
from curator.helpers.date_ops import parse_date_pattern
from curator.helpers.getters import get_indices
from . import CuratorTestCase
from . import testvars

HOST = os.environ.get('TEST_ES_SERVER', 'http://127.0.0.1:9200')
OLDINDEX = 'rolltome-000001'
NEWINDEX = 'rolltome-000002'
ALIAS = 'delamitri'


class TestActionFileRollover(CuratorTestCase):
    def test_max_age_true(self):
        condition = 'max_age'
        value = '1s'
        expected = {NEWINDEX: {'aliases': {ALIAS: {}}}}
        self.client.indices.create(index=OLDINDEX, aliases={ALIAS: {}})
        time.sleep(1)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_one.format(ALIAS, condition, value),
        )
        self.invoke_runner()
        assert expected == self.client.indices.get_alias(name=ALIAS)

    def test_max_age_false(self):
        condition = 'max_age'
        value = '10s'
        expected = {OLDINDEX: {'aliases': {ALIAS: {}}}}
        self.client.indices.create(index=OLDINDEX, aliases={ALIAS: {}})
        time.sleep(1)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_one.format(ALIAS, condition, value),
        )
        self.invoke_runner()
        assert expected == self.client.indices.get_alias(name=ALIAS)

    def test_max_docs_true(self):
        condition = 'max_docs'
        value = '2'
        expecto = {'aliases': {ALIAS: {}}}
        self.client.indices.create(index=OLDINDEX, aliases={ALIAS: {}})
        self.add_docs(OLDINDEX)
        self.client.indices.rollover(
            alias=ALIAS, conditions={condition: value}, dry_run=True
        )
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_one.format(ALIAS, condition, value),
        )
        self.invoke_runner()
        response = self.client.indices.get_alias(name=ALIAS)
        assert 1 == len(list(response.keys()))
        assert NEWINDEX == list(response.keys())[0]
        assert expecto == response[NEWINDEX]

    def test_max_docs_false(self):
        condition = 'max_docs'
        value = '5'
        expected = {OLDINDEX: {'aliases': {ALIAS: {}}}}
        self.client.indices.create(index=OLDINDEX, aliases={ALIAS: {}})
        self.add_docs(OLDINDEX)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_one.format(ALIAS, condition, value),
        )
        self.invoke_runner()
        assert expected == self.client.indices.get_alias(name=ALIAS)

    def test_conditions_both_false(self):
        max_age = '10s'
        max_docs = '5'
        expected = {OLDINDEX: {'aliases': {ALIAS: {}}}}
        self.client.indices.create(index=OLDINDEX, aliases={ALIAS: {}})
        self.add_docs(OLDINDEX)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_both.format(ALIAS, max_age, max_docs),
        )
        self.invoke_runner()
        assert expected == self.client.indices.get_alias(name=ALIAS)

    def test_conditions_both_true(self):
        max_age = '1s'
        max_docs = '2'
        expected = {NEWINDEX: {'aliases': {ALIAS: {}}}}
        self.client.indices.create(index=OLDINDEX, aliases={ALIAS: {}})
        time.sleep(1)
        self.add_docs(OLDINDEX)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_both.format(ALIAS, max_age, max_docs),
        )
        self.invoke_runner()
        assert expected == self.client.indices.get_alias(name=ALIAS)

    def test_conditions_one_false_one_true(self):
        max_age = '10s'
        max_docs = '2'
        expected = {NEWINDEX: {'aliases': {ALIAS: {}}}}
        self.client.indices.create(index=OLDINDEX, aliases={ALIAS: {}})
        self.add_docs(OLDINDEX)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_both.format(ALIAS, max_age, max_docs),
        )
        self.invoke_runner()
        assert expected == self.client.indices.get_alias(name=ALIAS)

    def test_conditions_one_empty_one_true(self):
        max_age = ' '
        max_docs = '2'
        expected = {OLDINDEX: {'aliases': {ALIAS: {}}}}
        self.client.indices.create(index=OLDINDEX, aliases={ALIAS: {}})
        self.add_docs(OLDINDEX)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_both.format(ALIAS, max_age, max_docs),
        )
        self.invoke_runner()
        assert expected == self.client.indices.get_alias(name=ALIAS)
        assert 1 == self.result.exit_code

    def test_bad_settings(self):
        max_age = '10s'
        max_docs = '2'
        expected = {OLDINDEX: {'aliases': {ALIAS: {}}}}
        self.client.indices.create(index=OLDINDEX, aliases={ALIAS: {}})
        self.add_docs(OLDINDEX)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_bad_settings.format(ALIAS, max_age, max_docs),
        )
        self.invoke_runner()
        assert expected == self.client.indices.get_alias(name=ALIAS)
        assert 1 == self.result.exit_code

    def test_extra_option(self):
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.bad_option_rollover_test.format('rollover'),
        )
        before = get_indices(self.client)
        self.invoke_runner()
        assert before == get_indices(self.client)
        assert 1 == self.result.exit_code

    def test_max_age_with_new_name(self):
        newindex = 'crazy_test'
        condition = 'max_age'
        value = '1s'
        expected = {newindex: {'aliases': {ALIAS: {}}}}
        self.client.indices.create(index=OLDINDEX, aliases={ALIAS: {}})
        time.sleep(1)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_with_name.format(ALIAS, condition, value, newindex),
        )
        self.invoke_runner()
        assert expected == self.client.indices.get_alias(name=ALIAS)

    def test_max_age_with_new_name_with_date(self):
        newindex = 'crazy_test-%Y.%m.%d'
        condition = 'max_age'
        value = '1s'
        expected = {parse_date_pattern(newindex): {'aliases': {ALIAS: {}}}}
        self.client.indices.create(index=OLDINDEX, aliases={ALIAS: {}})
        time.sleep(1)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_with_name.format(ALIAS, condition, value, newindex),
        )
        self.invoke_runner()
        assert expected == self.client.indices.get_alias(name=ALIAS)

    def test_max_age_old_index_with_date_with_new_index(self):
        oldindex = 'crazy_test-2017.01.01'
        newindex = 'crazy_test-%Y.%m.%d'
        condition = 'max_age'
        value = '1s'
        expected = {f"{parse_date_pattern(newindex)}": {'aliases': {ALIAS: {}}}}
        self.client.indices.create(index=oldindex, aliases={ALIAS: {}})
        time.sleep(1)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_with_name.format(ALIAS, condition, value, newindex),
        )
        self.invoke_runner()
        assert expected == self.client.indices.get_alias(name=ALIAS)

    def test_is_write_alias(self):
        condition = 'max_age'
        value = '1s'
        request_body = {'aliases': {ALIAS: {'is_write_index': True}}}
        expected = 2
        self.client.indices.create(index=OLDINDEX, aliases=request_body['aliases'])
        time.sleep(1)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_one.format(ALIAS, condition, value),
        )
        self.invoke_runner()
        assert expected == len(self.client.indices.get_alias(name=ALIAS))

    def test_no_rollover_ilm_associated(self):
        condition = 'max_age'
        value = '1s'
        expected = 1
        self.client.indices.create(
            index=OLDINDEX,
            settings={'index': {'lifecycle': {'name': 'generic'}}},
            aliases={ALIAS: {}},
        )
        time.sleep(1)
        self.write_config(self.args['configfile'], testvars.client_config.format(HOST))
        self.write_config(
            self.args['actionfile'],
            testvars.rollover_one.format(ALIAS, condition, value),
        )
        self.invoke_runner()
        assert 0 == self.result.exit_code
        assert expected == len(self.client.indices.get_alias(name=ALIAS))
        assert OLDINDEX == list(self.client.indices.get_alias(name=ALIAS).keys())[0]


class TestCLIRollover(CuratorTestCase):
    def test_max_age_true(self):
        value = '1s'
        expected = {NEWINDEX: {'aliases': {ALIAS: {}}}}
        self.client.indices.create(index=OLDINDEX, aliases={ALIAS: {}})
        time.sleep(1)
        args = self.get_runner_args()
        args += [
            '--config',
            self.args['configfile'],
            'rollover',
            '--name',
            ALIAS,
            '--max_age',
            value,
        ]
        assert 0 == self.run_subprocess(
            args, logname='TestCLIRollover.test_max_age_true'
        )
        assert expected == self.client.indices.get_alias(name=ALIAS)



"""test_helpers_date_ops"""

from datetime import datetime
from unittest import TestCase
from unittest.mock import Mock
import pytest
from elasticsearch9 import NotFoundError
from elastic_transport import ApiResponseMeta
from curator.exceptions import ConfigurationError
from curator.helpers.date_ops import (
    absolute_date_range,
    date_range,
    datetime_to_epoch,
    fix_epoch,
    get_date_regex,
    get_datemath,
    get_point_of_reference,
    isdatemath,
)


class TestGetDateRegex(TestCase):
    """TestGetDateRegex

    Test helpers.date_ops.get_date_regex functionality.
    """

    def test_non_escaped(self):
        """test_non_escaped

        Should return a proper regex from a non-escaped Python date string
        """
        assert '\\d{4}\\-\\d{2}\\-\\d{2}t\\d{2}' == get_date_regex('%Y-%m-%dt%H')


class TestFixEpoch(TestCase):
    """TestFixEpoch

    Test helpers.date_ops.fix_epoch functionality.
    """

    def test_fix_epoch(self):
        """test_fix_epoch

        Should return straight epoch time in seconds, removing milliseconds or more decimals
        """
        for long_epoch, epoch in [
            (1459287636, 1459287636),
            (14592876369, 14592876),
            (145928763699, 145928763),
            (1459287636999, 1459287636),
            (1459287636000000, 1459287636),
            (145928763600000000, 1459287636),
            (145928763600000001, 1459287636),
            (1459287636123456789, 1459287636),
        ]:
            assert epoch == fix_epoch(long_epoch)

    def test_fix_epoch_raise(self):
        """test_fix_epoch_raise

        Should raise a ``ValueError`` exception when an improper value is passed
        """
        with pytest.raises(ValueError):
            fix_epoch(None)


class TestGetPointOfReference(TestCase):
    """TestGetPointOfReference

    Test helpers.date_ops.get_point_of_reference functionality.
    """

    def test_get_point_of_reference(self):
        """test_get_point_of_reference

        Should return a reference point n units * seconds prior to the present epoch time
        """
        epoch = 1459288037
        for unit, result in [
            ('seconds', epoch - 1),
            ('minutes', epoch - 60),
            ('hours', epoch - 3600),
            ('days', epoch - 86400),
            ('weeks', epoch - (86400 * 7)),
            ('months', epoch - (86400 * 30)),
            ('years', epoch - (86400 * 365)),
        ]:
            # self.assertEqual(result, get_point_of_reference(unit, 1, epoch))
            assert result == get_point_of_reference(unit, 1, epoch)

    def test_get_por_raise(self):
        """test_get_por_raise

        Should raise a ``ValueError`` exception when an improper value is passed
        """
        self.assertRaises(ValueError, get_point_of_reference, 'invalid', 1)
        with pytest.raises(ValueError):
            get_point_of_reference('invalid', 1)


class TestDateRange(TestCase):
    """TestDateRange

    Test helpers.date_ops.date_range functionality.
    """

    EPOCH = datetime_to_epoch(datetime(2017, 4, 3, 22, 50, 17))

    def test_bad_unit(self):
        """test_bad_unit

        Should raise a ``ConfigurationError`` exception when an improper unit value is passed
        """
        with pytest.raises(ConfigurationError, match=r'"unit" must be one of'):
            date_range('invalid', 1, 1)

    def test_bad_range(self):
        """test_bad_range

        Should raise a ``ConfigurationError`` exception when an improper range value is passed
        """
        with pytest.raises(
            ConfigurationError, match=r'must be greater than or equal to'
        ):
            date_range('hours', 1, -1)

    def test_hours_single(self):
        """test_hours_single

        Should match hard-coded values when range_from = -1 and range_to = -1 and unit is hours
        """
        unit = 'hours'
        range_from = -1
        range_to = -1
        start = datetime_to_epoch(datetime(2017, 4, 3, 21, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 3, 21, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_hours_past_range(self):
        """test_hours_past_range

        Should match hard-coded values when range_from = -3 and range_to = -1 and unit is hours
        """
        unit = 'hours'
        range_from = -3
        range_to = -1
        start = datetime_to_epoch(datetime(2017, 4, 3, 19, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 3, 21, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_hours_future_range(self):
        """test_hours_future_range

        Should match hard-coded values when range_from = 0 and range_to = 2 and unit is hours
        """
        unit = 'hours'
        range_from = 0
        range_to = 2
        start = datetime_to_epoch(datetime(2017, 4, 3, 22, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 4, 00, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_hours_span_range(self):
        """test_hours_span_range

        Should match hard-coded values when range_from = -1 and range_to = 2 and unit is hours
        """
        unit = 'hours'
        range_from = -1
        range_to = 2
        start = datetime_to_epoch(datetime(2017, 4, 3, 21, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 4, 00, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_days_single(self):
        """test_days_single

        Should match hard-coded values when range_from = -1 and range_to = -1 and unit is days
        """
        unit = 'days'
        range_from = -1
        range_to = -1
        start = datetime_to_epoch(datetime(2017, 4, 2, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 2, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_days_past_range(self):
        """test_days_range

        Should match hard-coded values when range_from = -3 and range_to = -1 and unit is days
        """
        unit = 'days'
        range_from = -3
        range_to = -1
        start = datetime_to_epoch(datetime(2017, 3, 31, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 2, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_days_future_range(self):
        """test_days_future_range

        Should match hard-coded values when range_from = 0 and range_to = 2 and unit is days
        """
        unit = 'days'
        range_from = 0
        range_to = 2
        start = datetime_to_epoch(datetime(2017, 4, 3, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 5, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_days_span_range(self):
        """test_days_span_range

        Should match hard-coded values when range_from = -1 and range_to = 2 and unit is days
        """
        unit = 'days'
        range_from = -1
        range_to = 2
        start = datetime_to_epoch(datetime(2017, 4, 2, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 5, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_weeks_single(self):
        """test_weeks_single

        Should match hard-coded values when range_from = -1 and range_to = -1 and unit is weeks
        """
        unit = 'weeks'
        range_from = -1
        range_to = -1
        start = datetime_to_epoch(datetime(2017, 3, 26, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 1, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_weeks_past_range(self):
        """test_weeks_past_range

        Should match hard-coded values when range_from = -3 and range_to = -1 and unit is weeks
        """
        unit = 'weeks'
        range_from = -3
        range_to = -1
        start = datetime_to_epoch(datetime(2017, 3, 12, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 1, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_weeks_future_range(self):
        """test_weeks_future_range

        Should match hard-coded values when range_from = 0 and range_to = 2 and unit is weeks
        """
        unit = 'weeks'
        range_from = 0
        range_to = 2
        start = datetime_to_epoch(datetime(2017, 4, 2, 00, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 22, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_weeks_span_range(self):
        """test_weeks_span_range

        Should match hard-coded values when range_from = -1 and range_to = 2 and unit is weeks
        """
        unit = 'weeks'
        range_from = -1
        range_to = 2
        start = datetime_to_epoch(datetime(2017, 3, 26, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 22, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_weeks_single_iso(self):
        """test_weeks_single_iso

        Should match hard-coded values when range_from = -1 and range_to = -1, unit is weeks,
        and ``week_starts_on`` = ``monday``
        """
        unit = 'weeks'
        range_from = -1
        range_to = -1
        start = datetime_to_epoch(datetime(2017, 3, 27, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 2, 23, 59, 59))
        # pylint: disable=line-too-long
        assert (start, end) == date_range(
            unit, range_from, range_to, epoch=self.EPOCH, week_starts_on='monday'
        )

    def test_weeks_past_range_iso(self):
        """test_weeks_past_range_iso

        Should match hard-coded values when range_from = -3 and range_to = -1, unit is weeks,
        and ``week_starts_on`` = ``monday``
        """
        unit = 'weeks'
        range_from = -3
        range_to = -1
        start = datetime_to_epoch(datetime(2017, 3, 13, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 2, 23, 59, 59))
        # pylint: disable=line-too-long
        assert (start, end) == date_range(
            unit, range_from, range_to, epoch=self.EPOCH, week_starts_on='monday'
        )

    def test_weeks_future_range_iso(self):
        """test_weeks_future_range_iso

        Should match hard-coded values when range_from = 0 and range_to = 2, unit is weeks,
        and ``week_starts_on`` = ``monday``
        """
        unit = 'weeks'
        range_from = 0
        range_to = 2
        start = datetime_to_epoch(datetime(2017, 4, 3, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 23, 23, 59, 59))
        # pylint: disable=line-too-long
        assert (start, end) == date_range(
            unit, range_from, range_to, epoch=self.EPOCH, week_starts_on='monday'
        )

    def test_weeks_span_range_iso(self):
        """test_weeks_span_range_iso

        Should match hard-coded values when range_from = -1 and range_to = 2, unit is weeks,
        and ``week_starts_on`` = ``monday``
        """
        unit = 'weeks'
        range_from = -1
        range_to = 2
        start = datetime_to_epoch(datetime(2017, 3, 27, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 4, 23, 23, 59, 59))
        # pylint: disable=line-too-long
        assert (start, end) == date_range(
            unit, range_from, range_to, epoch=self.EPOCH, week_starts_on='monday'
        )

    def test_months_single(self):
        """test_months_single

        Should match hard-coded values when range_from = -1 and range_to = -1 and unit is months
        """
        unit = 'months'
        range_from = -1
        range_to = -1
        start = datetime_to_epoch(datetime(2017, 3, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 3, 31, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_months_past_range(self):
        """test_months_past_range

        Should match hard-coded values when range_from = -4 and range_to = -1 and unit is months
        """
        unit = 'months'
        range_from = -4
        range_to = -1
        start = datetime_to_epoch(datetime(2016, 12, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 3, 31, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_months_future_range(self):
        """test_months_future_range

        Should match hard-coded values when range_from = 7 and range_to = 10 and unit is months
        """
        unit = 'months'
        range_from = 7
        range_to = 10
        start = datetime_to_epoch(datetime(2017, 11, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2018, 2, 28, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_months_super_future_range(self):
        """test_months_super_future_range

        Should match hard-coded values when range_from = 9 and range_to = 10 and unit is months
        """
        unit = 'months'
        range_from = 9
        range_to = 10
        start = datetime_to_epoch(datetime(2018, 1, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2018, 2, 28, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_months_span_range(self):
        """test_months_span_range

        Should match hard-coded values when range_from = -1 and range_to = 2 and unit is months
        """
        unit = 'months'
        range_from = -1
        range_to = 2

        start = datetime_to_epoch(datetime(2017, 3, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 6, 30, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_years_single(self):
        """test_years_single

        Should match hard-coded values when range_from = -1 and range_to = -1 and unit is years
        """
        unit = 'years'
        range_from = -1
        range_to = -1
        start = datetime_to_epoch(datetime(2016, 1, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2016, 12, 31, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_years_past_range(self):
        """test_years_past_range

        Should match hard-coded values when range_from = -3 and range_to = -1 and unit is years
        """
        unit = 'years'
        range_from = -3
        range_to = -1
        start = datetime_to_epoch(datetime(2014, 1, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2016, 12, 31, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_years_future_range(self):
        """test_years_future_range

        Should match hard-coded values when range_from = 0 and range_to = 2 and unit is years
        """
        unit = 'years'
        range_from = 0
        range_to = 2
        start = datetime_to_epoch(datetime(2017, 1, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2019, 12, 31, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)

    def test_years_span_range(self):
        """test_years_span_range

        Should match hard-coded values when range_from = -1 and range_to = 2 and unit is years
        """
        unit = 'years'
        range_from = -1
        range_to = 2
        start = datetime_to_epoch(datetime(2016, 1, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2019, 12, 31, 23, 59, 59))
        assert (start, end) == date_range(unit, range_from, range_to, epoch=self.EPOCH)


class TestAbsoluteDateRange(TestCase):
    """TestAbsoluteDateRange

    Test helpers.date_ops.absolute_date_range functionality.
    """

    def test_bad_unit(self):
        """test_bad_unit

        Should raise a ``ConfigurationError`` exception when an invalid value for unit is passed
        """
        unit = 'invalid'
        date_from = '2017.01'
        date_from_format = '%Y.%m'
        date_to = '2017.01'
        date_to_format = '%Y.%m'
        with pytest.raises(ConfigurationError, match=r'"unit" must be one of'):
            absolute_date_range(
                unit, date_from, date_to, date_from_format, date_to_format
            )

    def test_bad_formats(self):
        """test_bad_formats

        Should raise a ``ConfigurationError`` exception when no value for ``date_from_format`` or
        ``date_to_format`` is passed.
        """
        unit = 'days'
        with pytest.raises(
            ConfigurationError,
            match=r'Must provide "date_from_format" and "date_to_format"',
        ):
            absolute_date_range(unit, 'meh', 'meh', None, 'meh')
        with pytest.raises(
            ConfigurationError,
            match=r'Must provide "date_from_format" and "date_to_format"',
        ):
            absolute_date_range(unit, 'meh', 'meh', 'meh', None)

    def test_bad_dates(self):
        """test_bad_dates

        Should raise a ``ConfigurationError`` exception when date formats cannot be parsed for
        ``date_from_format`` and ``date_to_format``
        """
        unit = 'weeks'
        date_from_format = '%Y.%m'
        date_to_format = '%Y.%m'
        with pytest.raises(ConfigurationError, match=r'Unable to parse "date_from"'):
            absolute_date_range(
                unit, 'meh', '2017.01', date_from_format, date_to_format
            )
        with pytest.raises(ConfigurationError, match=r'Unable to parse "date_to"'):
            absolute_date_range(
                unit, '2017.01', 'meh', date_from_format, date_to_format
            )

    def test_single_month(self):
        """test_single_month

        Output should match hard-coded values
        """
        unit = 'months'
        date_from = '2017.01'
        date_from_format = '%Y.%m'
        date_to = '2017.01'
        date_to_format = '%Y.%m'
        start = datetime_to_epoch(datetime(2017, 1, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 1, 31, 23, 59, 59))
        result = absolute_date_range(
            unit, date_from, date_to, date_from_format, date_to_format
        )
        assert (start, end) == result

    def test_multiple_month(self):
        """test_multiple_month

        Output should match hard-coded values
        """
        unit = 'months'
        date_from = '2016.11'
        date_from_format = '%Y.%m'
        date_to = '2016.12'
        date_to_format = '%Y.%m'
        start = datetime_to_epoch(datetime(2016, 11, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2016, 12, 31, 23, 59, 59))
        result = absolute_date_range(
            unit, date_from, date_to, date_from_format, date_to_format
        )
        assert (start, end) == result

    def test_single_year(self):
        """test_single_year

        Output should match hard-coded values
        """
        unit = 'years'
        date_from = '2017'
        date_from_format = '%Y'
        date_to = '2017'
        date_to_format = '%Y'
        start = datetime_to_epoch(datetime(2017, 1, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 12, 31, 23, 59, 59))
        result = absolute_date_range(
            unit, date_from, date_to, date_from_format, date_to_format
        )
        assert (start, end) == result

    def test_multiple_year(self):
        """test_multiple_year

        Output should match hard-coded values
        """
        unit = 'years'
        date_from = '2016'
        date_from_format = '%Y'
        date_to = '2017'
        date_to_format = '%Y'
        start = datetime_to_epoch(datetime(2016, 1, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 12, 31, 23, 59, 59))
        result = absolute_date_range(
            unit, date_from, date_to, date_from_format, date_to_format
        )
        assert (start, end) == result

    def test_single_week_uw(self):
        """test_single_week_UW

        Output should match hard-coded values
        """
        unit = 'weeks'
        date_from = '2017-01'
        date_from_format = '%Y-%U'
        date_to = '2017-01'
        date_to_format = '%Y-%U'
        start = datetime_to_epoch(datetime(2017, 1, 2, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 1, 8, 23, 59, 59))
        result = absolute_date_range(
            unit, date_from, date_to, date_from_format, date_to_format
        )
        assert (start, end) == result

    def test_multiple_weeks_uw(self):
        """test_multiple_weeks_UW

        Output should match hard-coded values
        """
        unit = 'weeks'
        date_from = '2017-01'
        date_from_format = '%Y-%U'
        date_to = '2017-04'
        date_to_format = '%Y-%U'
        start = datetime_to_epoch(datetime(2017, 1, 2, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 1, 29, 23, 59, 59))
        result = absolute_date_range(
            unit, date_from, date_to, date_from_format, date_to_format
        )
        assert (start, end) == result

    def test_single_week_iso(self):
        """test_single_week_ISO

        Output should match hard-coded values
        """
        unit = 'weeks'
        date_from = '2014-01'
        date_from_format = '%G-%V'
        date_to = '2014-01'
        date_to_format = '%G-%V'
        start = datetime_to_epoch(datetime(2013, 12, 30, 0, 0, 0))
        end = datetime_to_epoch(datetime(2014, 1, 5, 23, 59, 59))
        result = absolute_date_range(
            unit, date_from, date_to, date_from_format, date_to_format
        )
        assert (start, end) == result

    def test_multiple_weeks_iso(self):
        """test_multiple_weeks_ISO

        Output should match hard-coded values
        """
        unit = 'weeks'
        date_from = '2014-01'
        date_from_format = '%G-%V'
        date_to = '2014-04'
        date_to_format = '%G-%V'
        start = datetime_to_epoch(datetime(2013, 12, 30, 0, 0, 0))
        end = datetime_to_epoch(datetime(2014, 1, 26, 23, 59, 59))
        result = absolute_date_range(
            unit, date_from, date_to, date_from_format, date_to_format
        )
        assert (start, end) == result

    def test_single_day(self):
        """test_single_day

        Output should match hard-coded values
        """
        unit = 'days'
        date_from = '2017.01.01'
        date_from_format = '%Y.%m.%d'
        date_to = '2017.01.01'
        date_to_format = '%Y.%m.%d'
        start = datetime_to_epoch(datetime(2017, 1, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 1, 1, 23, 59, 59))
        result = absolute_date_range(
            unit, date_from, date_to, date_from_format, date_to_format
        )
        assert (start, end) == result

    def test_multiple_days(self):
        """test_multiple_days

        Output should match hard-coded values
        """
        unit = 'days'
        date_from = '2016.12.31'
        date_from_format = '%Y.%m.%d'
        date_to = '2017.01.01'
        date_to_format = '%Y.%m.%d'
        start = datetime_to_epoch(datetime(2016, 12, 31, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 1, 1, 23, 59, 59))
        result = absolute_date_range(
            unit, date_from, date_to, date_from_format, date_to_format
        )
        assert (start, end) == result

    def test_iso8601(self):
        """test_ISO8601

        Output should match hard-coded values
        """
        unit = 'seconds'
        date_from = '2017-01-01T00:00:00'
        date_from_format = '%Y-%m-%dT%H:%M:%S'
        date_to = '2017-01-01T12:34:56'
        date_to_format = '%Y-%m-%dT%H:%M:%S'
        start = datetime_to_epoch(datetime(2017, 1, 1, 0, 0, 0))
        end = datetime_to_epoch(datetime(2017, 1, 1, 12, 34, 56))
        result = absolute_date_range(
            unit, date_from, date_to, date_from_format, date_to_format
        )
        assert (start, end) == result


class TestIsDateMath(TestCase):
    """TestIsDateMath

    Test helpers.date_ops.isdatemath functionality.
    """

    def test_positive(self):
        """test_positive

        Result should match hard-coded sample
        """
        data = '<encapsulated>'
        assert isdatemath(data)

    def test_negative(self):
        """test_negative

        Result should not match hard-coded sample
        """
        data = 'not_encapsulated'
        assert not isdatemath(data)

    def test_raises(self):
        """test_raises

        Should raise ConfigurationError exception when malformed data is passed
        """
        data = '<badly_encapsulated'
        with pytest.raises(
            ConfigurationError, match=r'Incomplete datemath encapsulation'
        ):
            isdatemath(data)


class TestGetDateMath(TestCase):
    """TestGetDateMath

    Test helpers.date_ops.get_datemath functionality.
    """

    def test_success(self):
        """test_success

        Result should match hard-coded sample
        """
        client = Mock()
        datemath = '{hasthemath}'
        psuedo_random = 'not_random_at_all'
        expected = 'curator_get_datemath_function_' + psuedo_random + '-hasthemath'
        # 5 positional args for meta: status, http_version, headers, duration, node
        meta = ApiResponseMeta(404, '1.1', {}, 0.01, None)
        body = {'error': {'index': expected}}
        msg = 'index_not_found_exception'
        # 3 positional args for NotFoundError: message, meta, body
        effect = NotFoundError(msg, meta, body)
        client.indices.get.side_effect = effect
        self.assertEqual('hasthemath', get_datemath(client, datemath, psuedo_random))

    def test_failure(self):
        """test_failure

        Should raise ConfigurationError exception when index is not found
        """
        client = Mock()
        datemath = '{hasthemath}'
        # 5 positional args for meta: status, http_version, headers, duration, node
        meta = ApiResponseMeta(404, '1.1', {}, 0.01, None)
        body = {'error': {'index': 'this_will_not_be_found'}}
        msg = 'index_not_found_exception'
        # 3 positional args for NotFoundError: message, meta, body
        effect = NotFoundError(msg, meta, body)
        client.indices.get.side_effect = effect
        self.assertRaises(ConfigurationError, get_datemath, client, datemath)



"""
Test cold2frozen functionality

This cannot work with the open-source release, so any testing must be manual
"""

# pylint: disable=missing-function-docstring, missing-class-docstring, line-too-long
# import os
# from time import sleep
# from curator import IndexList
# from curator.actions import Cold2Frozen, Snapshot
# from curator.exceptions import (
#     CuratorException, FailedExecution, SearchableSnapshotException)
# from curator.helpers.getters import get_snapshot
# from . import CuratorTestCase
# from . import testvars

# HOST = os.environ.get('TEST_ES_SERVER', 'http://127.0.0.1:9200')
# '      repository: {0}\n'
# '      - filtertype: {1}\n'
# '        source: {2}\n'
# '        direction: {3}\n'
# '        timestring: {4}\n'
# '        unit: {5}\n'
# '        unit_count: {6}\n'
# '        epoch: {7}\n')

# class TestActionFileCold2Frozen(CuratorTestCase):
#     SNAP = 'cold2frozen'
#     ORIG = 'myindex-000001'
#     ALIAS = 'myindex'
#     AL1 = [{'add': {'index': ORIG, 'alias': ALIAS}}]
#     AL2 = [
#         {'remove': {'index': ORIG, 'alias': ALIAS}},
#         {'add': {'index': f'restored-{ORIG}', 'alias': ALIAS}}
#     ]
#     COLD = {'routing': {'allocation': {'include': {'_tier_preference': 'data_cold'}}}}
#     def test_cold2frozen1(self):
#         ### Create snapshots to delete and verify them
#         self.create_repository()
#         self.add_docs(self.ORIG)
#         # Assign an alias
#         self.client.indices.update_aliases(actions=self.AL1)
#         # Forcemerge the index down
#         self.client.indices.forcemerge(index=self.ORIG, max_num_segments=1)
#         # Get an indexlist object
#         ilo = IndexList(self.client)
#         # Snapshot the single index in it
#         snp = Snapshot(
#             ilo, repository=self.args['repository'], name=self.SNAP,
#             wait_interval=0.5
#         )
#         snp.do_action()
#         # Verify the snapshot
#         snapshot = get_snapshot(self.client, self.args['repository'], self.SNAP)
#         assert 1 == len(snapshot['snapshots'])
#         sleep(2.0)
#         # Mount the index in the snapshot in the cold tier
#         self.client.searchable_snapshots.mount(
#             repository=self.args['repository'], snapshot=self.SNAP, index=self.ORIG,
#             renamed_index=f'restored-{self.ORIG}', index_settings=self.COLD,
#             storage='full_copy')
#         # Update the new index to have the aliases of the original one
#         self.client.indices.update_aliases(actions=self.AL2)
#         # Delete the original index
#         self.client.indices.delete(index=self.ORIG)
#         # Get a new indexlist object
#         ilo = IndexList(self.client)
#         # Verify it only has one index in it
#         assert 1 == len(ilo.indices)
#         # ...and that the index is our restored- cold-tier mount
#         settings = self.client.indices.get(
#             index=f'restored-{self.ORIG}')[f'restored-{self.ORIG}']
#         # ...and that it has our alias
#         assert {self.ALIAS: {}} == settings['aliases']
#         # ...and that it has the appropriate settings
#         snapchk = settings['settings']['index']['store']['snapshot']
#         assert self.SNAP == snapchk['snapshot_name']
#         assert self.ORIG == snapchk['index_name']
#         assert self.args['repository'] == snapchk['repository_name']



#!/usr/bin/env python3
import shutil
from platform import machine, system, python_version

MAJOR, MINOR = tuple(python_version().split('.')[:-1])
SYSTEM = system().lower()
BUILD = f'build/exe.{system().lower()}-{machine()}-{MAJOR}.{MINOR}'
TARGET = 'curator_build'

# Rename the path of BUILD to be generic enough for Dockerfile to get
# In other words, rename it to 'curator_build'
shutil.move(BUILD, TARGET)



"""test_action_rollover"""
# pylint: disable=missing-function-docstring, missing-class-docstring, protected-access, attribute-defined-outside-init
from unittest import TestCase
from unittest.mock import Mock
from curator.actions import Rollover
from curator.exceptions import ConfigurationError

# Get test variables and constants from a single source
from . import testvars

class TestActionRollover(TestCase):
    VERSION = {'version': {'number': '8.0.0'} }
    def builder(self):
        self.client = Mock()
        self.client.info.return_value = self.VERSION
    def test_init_raise_bad_client(self):
        self.assertRaises(TypeError, Rollover, 'invalid', 'name', {})
    def test_init_raise_bad_conditions(self):
        self.builder()
        self.assertRaises(ConfigurationError, Rollover, self.client, 'name', 'string')
    def test_init_raise_bad_extra_settings(self):
        self.builder()
        self.assertRaises(
            ConfigurationError, Rollover, self.client, 'name', {'a':'b'}, None, 'string')
    def test_init_raise_non_rollable_index(self):
        self.builder()
        self.client.indices.get_alias.return_value = testvars.alias_retval
        self.assertRaises(ValueError, Rollover, self.client, testvars.named_alias, {'a':'b'})
    def test_do_dry_run(self):
        self.builder()
        self.client.indices.get_alias.return_value = testvars.rollable_alias
        self.client.indices.rollover.return_value = testvars.dry_run_rollover
        rlo = Rollover(self.client, testvars.named_alias, testvars.rollover_conditions)
        self.assertIsNone(rlo.do_dry_run())
    def test_max_size_in_acceptable_verion(self):
        self.builder()
        self.client.indices.get_alias.return_value = testvars.rollable_alias
        conditions = { 'max_size': '1g' }
        rlo = Rollover(self.client, testvars.named_alias, conditions)
        self.assertEqual(conditions, rlo.conditions)



"""test_action_delete_indices"""
# pylint: disable=missing-function-docstring, missing-class-docstring, protected-access, attribute-defined-outside-init
from unittest import TestCase
from unittest.mock import Mock
from curator.actions import DeleteIndices
from curator.exceptions import FailedExecution
from curator import IndexList
# Get test variables and constants from a single source
from . import testvars

class TestActionDeleteIndices(TestCase):
    VERSION = {'version': {'number': '8.0.0'} }
    def builder(self):
        self.client = Mock()
        self.client.info.return_value = self.VERSION
        self.client.cat.indices.return_value = testvars.state_one
        self.client.indices.get_settings.return_value = testvars.settings_one
        self.client.indices.stats.return_value = testvars.stats_one
        self.client.indices.exists_alias.return_value = False
        self.ilo = IndexList(self.client)
    def builder4(self):
        self.client = Mock()
        self.client.info.return_value = self.VERSION
        self.client.cat.indices.return_value = testvars.state_four
        self.client.indices.get_settings.return_value = testvars.settings_four
        self.client.indices.stats.return_value = testvars.stats_four
        self.client.indices.exists_alias.return_value = False
        self.client.indices.delete.return_value = None
        self.ilo = IndexList(self.client)
    def test_init_raise(self):
        self.assertRaises(TypeError, DeleteIndices, 'invalid')
    def test_init_raise_bad_master_timeout(self):
        self.builder()
        self.assertRaises(TypeError, DeleteIndices, self.ilo, 'invalid')
    def test_init(self):
        self.builder()
        dio = DeleteIndices(self.ilo)
        self.assertEqual(self.ilo, dio.index_list)
        self.assertEqual(self.client, dio.client)
    def test_do_dry_run(self):
        self.builder4()
        dio = DeleteIndices(self.ilo)
        self.assertIsNone(dio.do_dry_run())
    def test_do_action(self):
        self.builder4()
        dio = DeleteIndices(self.ilo)
        self.assertIsNone(dio.do_action())
    def test_do_action_not_successful(self):
        self.builder4()
        dio = DeleteIndices(self.ilo)
        self.assertIsNone(dio.do_action())
    def test_do_action_raises_exception(self):
        self.builder4()
        self.client.indices.delete.side_effect = testvars.fake_fail
        dio = DeleteIndices(self.ilo)
        self.assertRaises(FailedExecution, dio.do_action)
    def test_verify_result_positive(self):
        self.builder4()
        dio = DeleteIndices(self.ilo)
        self.assertTrue(dio._verify_result([],2))



"""Show Index/Snapshot Singletons"""

from datetime import datetime
import click
from curator.cli_singletons.object_class import CLIAction
from curator.cli_singletons.utils import validate_filter_json
from curator.helpers.getters import byte_size
from curator.defaults.settings import footer
from curator._version import __version__

# ### Indices ###


# pylint: disable=line-too-long
@click.command(
    epilog=footer(__version__, tail='singleton-cli.html#_show_indicessnapshots')
)
@click.option(
    '--search_pattern', type=str, default='*', help='Elasticsearch Index Search Pattern'
)
@click.option('--verbose', help='Show verbose output.', is_flag=True, show_default=True)
@click.option(
    '--header', help='Print header if --verbose', is_flag=True, show_default=True
)
@click.option(
    '--epoch', help='Print time as epoch if --verbose', is_flag=True, show_default=True
)
@click.option(
    '--ignore_empty_list',
    is_flag=True,
    help='Do not raise exception if there are no actionable indices',
)
@click.option(
    '--allow_ilm_indices/--no-allow_ilm_indices',
    help='Allow Curator to operate on Index Lifecycle Management monitored indices.',
    default=False,
    show_default=True,
)
@click.option(
    '--include_hidden/--no-include_hidden',
    help='Allow Curator to operate on hidden indices (and data_streams).',
    default=False,
    show_default=True,
)
@click.option(
    '--filter_list',
    callback=validate_filter_json,
    default='{"filtertype":"none"}',
    help='JSON string representing an array of filters.',
)
@click.pass_context
def show_indices(
    ctx,
    search_pattern,
    verbose,
    header,
    epoch,
    ignore_empty_list,
    allow_ilm_indices,
    include_hidden,
    filter_list,
):
    """
    Show Indices
    """
    # ctx.info_name is the name of the function or name specified in
    # @click.command decorator
    action = CLIAction(
        'show_indices',
        ctx.obj['configdict'],
        {
            'search_pattern': search_pattern,
            'allow_ilm_indices': allow_ilm_indices,
            'include_hidden': include_hidden,
        },
        filter_list,
        ignore_empty_list,
    )
    action.get_list_object()
    action.do_filters()
    indices = sorted(action.list_object.indices)
    # Do some calculations to figure out the proper column sizes
    allbytes = []
    alldocs = []
    for idx in indices:
        allbytes.append(byte_size(action.list_object.index_info[idx]['size_in_bytes']))
        alldocs.append(str(action.list_object.index_info[idx]['docs']))
    if epoch:
        timeformat = '{6:>13}'
        column = 'creation_date'
    else:
        timeformat = '{6:>20}'
        column = 'Creation Timestamp'
    formatting = (
        '{0:' + str(len(max(indices, key=len))) + '} '
        '{1:>5} '
        '{2:>' + str(len(max(allbytes, key=len)) + 1) + '} '
        '{3:>' + str(len(max(alldocs, key=len)) + 1) + '} '
        '{4:>3} {5:>3} ' + timeformat
    )
    # Print the header, if both verbose and header are enabled
    if header and verbose:
        click.secho(
            formatting.format('Index', 'State', 'Size', 'Docs', 'Pri', 'Rep', column),
            bold=True,
            underline=True,
        )
    # Loop through indices and print info, if verbose
    for idx in indices:
        data = action.list_object.index_info[idx]
        if verbose:
            if epoch:
                datefield = (
                    data['age']['creation_date']
                    if 'creation_date' in data['age']
                    else 0
                )
            else:
                datefield = (
                    datetime.utcfromtimestamp(data['age']['creation_date']).isoformat()
                    if 'creation_date' in data['age']
                    else 'unknown/closed'
                )
            click.echo(
                formatting.format(
                    idx,
                    data['state'],
                    byte_size(data['size_in_bytes']),
                    data['docs'],
                    data['number_of_shards'],
                    data['number_of_replicas'],
                    f'{datefield}Z',
                )
            )
        else:
            click.secho(f'{idx}')


# ### Snapshots ###


# pylint: disable=line-too-long
@click.command(
    epilog=footer(__version__, tail='singleton-cli.html#_show_indicessnapshots')
)
@click.option('--repository', type=str, required=True, help='Snapshot repository name')
@click.option(
    '--ignore_empty_list',
    is_flag=True,
    help='Do not raise exception if there are no actionable snapshots',
)
@click.option(
    '--filter_list',
    callback=validate_filter_json,
    default='{"filtertype":"none"}',
    help='JSON string representing an array of filters.',
)
@click.pass_context
def show_snapshots(ctx, repository, ignore_empty_list, filter_list):
    """
    Show Snapshots
    """
    # ctx.info_name is the name of the function or name specified in
    # @click.command decorator
    action = CLIAction(
        'show_snapshots',
        ctx.obj['configdict'],
        {},
        filter_list,
        ignore_empty_list,
        repository=repository,
    )
    action.get_list_object()
    action.do_filters()
    for snapshot in sorted(action.list_object.snapshots):
        click.secho(f'{snapshot}')



"""Allocation action class"""

import logging

# pylint: disable=import-error
from curator.exceptions import MissingArgument
from curator.helpers.testers import verify_index_list
from curator.helpers.waiters import wait_for_it
from curator.helpers.utils import chunk_index_list, report_failure, show_dry_run, to_csv


class Allocation:
    """Allocation Action Class"""

    def __init__(
        self,
        ilo,
        key=None,
        value=None,
        allocation_type='require',
        wait_for_completion=False,
        wait_interval=3,
        max_wait=-1,
    ):
        """
        :param ilo: An IndexList Object
        :param key: An arbitrary metadata attribute key.  Must match the key
            assigned to at least one node.
        :param value: An arbitrary metadata attribute value.  Must correspond to
            values associated with at least one node.
        :param allocation_type: Type of allocation to apply. Default is ``require``
        :param wait_for_completion: Wait for completion before returning.
        :param wait_interval: Seconds to wait between completion checks.
        :param max_wait: Maximum number of seconds to ``wait_for_completion``

        :type ilo: :py:class:`~.curator.indexlist.IndexList`
        :type key: str
        :type value: str
        :type allocation_type: str
        :type wait_for_completion: bool
        :type wait_interval: int
        :type max_wait: int

        .. note::
            See more about `shard allocation filtering
            </https://www.elastic.co/guide/en/elasticsearch/reference/8.6/shard-allocation-filtering.html>`_.
        """
        verify_index_list(ilo)
        if not key:
            raise MissingArgument('No value for "key" provided')
        if allocation_type not in ['require', 'include', 'exclude']:
            raise ValueError(
                f'{allocation_type} is an invalid allocation_type.  Must be one of '
                f'"require", "include", "exclude".'
            )
        #: The :py:class:`~.curator.indexlist.IndexList` object passed as ``ilo``
        self.index_list = ilo
        #: The :py:class:`~.elasticsearch.Elasticsearch` client object derived from
        #: :py:attr:`index_list`
        self.client = ilo.client
        self.loggit = logging.getLogger('curator.actions.allocation')
        bkey = f'index.routing.allocation.{allocation_type}.{key}'
        #: Populated at instance creation time. Value is built from the passed params
        #: ``allocation_type``, ``key``, and ``value``, e.g.
        #: ``index.routing.allocation.allocation_type.key.value``
        self.settings = {bkey: value}
        #: Object attribute that gets the value of param ``wait_for_completion``
        self.wfc = wait_for_completion
        #: Object attribute that gets the value of param ``wait_interval``
        self.wait_interval = wait_interval
        #: Object attribute that gets the value of param ``max_wait``
        self.max_wait = max_wait

    def do_dry_run(self):
        """Log what the output would be, but take no action."""
        show_dry_run(self.index_list, 'allocation', settings=self.settings)

    def do_action(self):
        """
        :py:meth:`~.elasticsearch.client.IndicesClient.put_settings` to indices in
        :py:attr:`index_list` with :py:attr:`settings`.
        """
        self.loggit.debug(
            'Cannot get change shard routing allocation of closed indices.  '
            'Omitting any closed indices.'
        )
        self.index_list.filter_closed()
        self.index_list.empty_list_check()
        self.loggit.info(
            'Updating %s selected indices: %s',
            len(self.index_list.indices),
            self.index_list.indices,
        )
        self.loggit.info('Updating index setting %s', self.settings)
        try:
            index_lists = chunk_index_list(self.index_list.indices)
            for lst in index_lists:
                self.client.indices.put_settings(
                    index=to_csv(lst), settings=self.settings
                )
                if self.wfc:
                    self.loggit.debug(
                        'Waiting for shards to complete relocation for indices: %s',
                        to_csv(lst),
                    )
                    wait_for_it(
                        self.client,
                        'allocation',
                        wait_interval=self.wait_interval,
                        max_wait=self.max_wait,
                    )
        # pylint: disable=broad-except
        except Exception as err:
            report_failure(err)



"""Alias unit tests"""
# pylint: disable=missing-function-docstring, missing-class-docstring, invalid-name, line-too-long, attribute-defined-outside-init
from unittest import TestCase
from unittest.mock import Mock
from curator import IndexList
from curator.exceptions import MissingArgument
from curator.actions.allocation import Allocation
from . import testvars

class TestActionAllocation(TestCase):
    VERSION = {'version': {'number': '5.0.0'} }
    def builder(self):
        self.client = Mock()
        self.client.info.return_value = self.VERSION
        self.client.cat.indices.return_value = testvars.state_one
        self.client.indices.get_settings.return_value = testvars.settings_one
        self.client.indices.stats.return_value = testvars.stats_one
        self.client.indices.exists_alias.return_value = False
        self.client.indices.put_settings.return_value = None
        self.ilo = IndexList(self.client)
    def test_init_raise(self):
        self.assertRaises(TypeError, Allocation, 'invalid')
    def test_init(self):
        self.builder()
        ao = Allocation(self.ilo, key='key', value='value')
        self.assertEqual(self.ilo, ao.index_list)
        self.assertEqual(self.client, ao.client)
    def test_create_body_no_key(self):
        self.builder()
        self.assertRaises(MissingArgument, Allocation, self.ilo)
    def test_create_body_invalid_allocation_type(self):
        self.builder()
        self.assertRaises(
            ValueError,
            Allocation, self.ilo,
            key='key', value='value', allocation_type='invalid'
        )
    def test_create_body_valid(self):
        self.builder()
        ao = Allocation(self.ilo, key='key', value='value')
        self.assertEqual({'index.routing.allocation.require.key': 'value'}, ao.settings)
    def test_do_action_raise_on_put_settings(self):
        self.builder()
        self.client.indices.put_settings.side_effect = testvars.fake_fail
        ao = Allocation(self.ilo, key='key', value='value')
        self.assertRaises(Exception, ao.do_action)
    def test_do_dry_run(self):
        self.builder()
        alo = Allocation(self.ilo, key='key', value='value')
        self.assertIsNone(alo.do_dry_run())
    def test_do_action(self):
        self.builder()
        alo = Allocation(self.ilo, key='key', value='value')
        self.assertIsNone(alo.do_action())
    def test_do_action_wait_v50(self):
        self.builder()
        self.client.cluster.health.return_value = {'relocating_shards':0}
        alo = Allocation(
            self.ilo, key='key', value='value', wait_for_completion=True)
        self.assertIsNone(alo.do_action())
    def test_do_action_wait_v51(self):
        self.builder()
        self.client.info.return_value = {'version': {'number': '5.1.1'} }
        self.client.cluster.health.return_value = {'relocating_shards':0}
        alo = Allocation(
            self.ilo, key='key', value='value', wait_for_completion=True)
        self.assertIsNone(alo.do_action())



"""test_action_delete_snapshots"""
from unittest import TestCase
from unittest.mock import Mock
from curator.actions import DeleteSnapshots
from curator.exceptions import FailedExecution
from curator import SnapshotList
# Get test variables and constants from a single source
from . import testvars as testvars

class TestActionDeleteSnapshots(TestCase):
    def test_init_raise(self):
        self.assertRaises(TypeError, DeleteSnapshots, 'invalid')
    def test_init(self):
        client = Mock()
        client.snapshot.get.return_value = testvars.snapshots
        client.snapshot.get_repository.return_value = testvars.test_repo
        slo = SnapshotList(client, repository=testvars.repo_name)
        do = DeleteSnapshots(slo)
        self.assertEqual(slo, do.snapshot_list)
        self.assertEqual(client, do.client)
    def test_do_dry_run(self):
        client = Mock()
        client.snapshot.get.return_value = testvars.snapshots
        client.snapshot.get_repository.return_value = testvars.test_repo
        client.tasks.get.return_value = testvars.no_snap_tasks
        client.snapshot.delete.return_value = None
        slo = SnapshotList(client, repository=testvars.repo_name)
        do = DeleteSnapshots(slo)
        self.assertIsNone(do.do_dry_run())
    def test_do_action(self):
        client = Mock()
        client.snapshot.get.return_value = testvars.snapshots
        client.snapshot.get_repository.return_value = testvars.test_repo
        client.tasks.list.return_value = testvars.no_snap_tasks
        client.snapshot.delete.return_value = None
        slo = SnapshotList(client, repository=testvars.repo_name)
        do = DeleteSnapshots(slo)
        self.assertIsNone(do.do_action())
    def test_do_action_raises_exception(self):
        client = Mock()
        client.snapshot.get.return_value = testvars.snapshots
        client.snapshot.get_repository.return_value = testvars.test_repo
        client.snapshot.delete.return_value = None
        client.tasks.list.return_value = testvars.no_snap_tasks
        client.snapshot.delete.side_effect = testvars.fake_fail
        slo = SnapshotList(client, repository=testvars.repo_name)
        do = DeleteSnapshots(slo)
        self.assertRaises(FailedExecution, do.do_action)
    ### This check is not necessary after ES 7.16 as it is possible to have
    ### up to 1000 concurrent snapshots
    ###
    ### https://www.elastic.co/guide/en/elasticsearch/reference/8.6/snapshot-settings.html
    ### snapshot.max_concurrent_operations
    ### (Dynamic, integer) Maximum number of concurrent snapshot operations. Defaults to 1000.
    ###
    ### This limit applies in total to all ongoing snapshot creation, cloning, and deletion
    ### operations. Elasticsearch will reject any operations that would exceed this limit.
    # def test_not_safe_to_snap_raises_exception(self):
    #     client = Mock()
    #     client.snapshot.get.return_value = testvars.inprogress
    #     client.snapshot.get_repository.return_value = testvars.test_repo
    #     client.tasks.list.return_value = testvars.no_snap_tasks
    #     slo = SnapshotList(client, repository=testvars.repo_name)
    #     do = DeleteSnapshots(slo, retry_interval=0, retry_count=1)
    #     self.assertRaises(curator.FailedExecution, do.do_action)
